@@ -5,38 +5,28 @@ from zope.interface import implements
 from twisted.internet import defer
 from foolscap.api import fireEventually
 import simplejson
-from allmydata.mutable.common import NotMutableError
+from allmydata.mutable.common import NotWriteableError
 from allmydata.mutable.filenode import MutableFileNode
-from allmydata.unknown import UnknownNode
-from allmydata.interfaces import IMutableFileNode, IDirectoryNode,\
-     IURI, IFileNode, IMutableFileURI, IFilesystemNode, \
+from allmydata.unknown import UnknownNode, strip_prefix_for_ro
+from allmydata.interfaces import IFilesystemNode, IDirectoryNode, IFileNode, \
+     IImmutableFileNode, IMutableFileNode, \
      ExistingChildError, NoSuchChildError, ICheckable, IDeepCheckable, \
-     CannotPackUnknownNodeError
+     MustBeDeepImmutableError, CapConstraintError
 from allmydata.check_results import DeepCheckResults, \
      DeepCheckAndRepairResults
 from allmydata.monitor import Monitor
 from allmydata.util import hashutil, mathutil, base32, log
-from allmydata.util.assertutil import _assert, precondition
+from allmydata.util.assertutil import precondition
 from allmydata.util.netstring import netstring, split_netstring
-from allmydata.uri import DirectoryURI, LiteralFileURI, from_string
+from allmydata.util.consumer import download_to_data
+from allmydata.uri import LiteralFileURI, from_string, wrap_dirnode_cap
 from pycryptopp.cipher.aes import AES
+from allmydata.util.dictutil import AuxValueDict
 
-class CachingDict(dict):
-    def __init__(self, *args):
-        super(CachingDict, self).__init__(*args)
-        self.serialized = {}
 
-    def __setitem__(self, k, v):
-        super(CachingDict, self).__setitem__(k, v)
-        self.serialized[k] = None
-
-    def get_both_items(self, k):
-        return (self.serialized.setdefault(k, None),
-                super(CachingDict, self).__getitem__(k))
-
-    def set_both_items(self, key, serialized, t):
-        self.serialized[key] = serialized
-        super(CachingDict, self).__setitem__(key, t)
+# TODO: {Deleter,MetadataSetter,Adder}.modify all start by unpacking the
+# contents and end by repacking them. It might be better to apply them to
+# the unpacked contents.
 
 class Deleter:
     def __init__(self, node, name, must_exist=True):
@@ -55,6 +45,7 @@ class Deleter:
         new_contents = self.node._pack_contents(children)
         return new_contents
 
+
 class MetadataSetter:
     def __init__(self, node, name, metadata):
         self.node = node
@@ -74,27 +65,27 @@ class Adder:
     def __init__(self, node, entries=None, overwrite=True):
         self.node = node
         if entries is None:
-            entries = []
+            entries = {}
+        precondition(isinstance(entries, dict), entries)
         self.entries = entries
         self.overwrite = overwrite
 
     def set_node(self, name, node, metadata):
         precondition(isinstance(name, unicode), name)
         precondition(IFilesystemNode.providedBy(node), node)
-        self.entries.append( [name, node, metadata] )
+        self.entries[name] = (node, metadata)
 
     def modify(self, old_contents, servermap, first_time):
         children = self.node._unpack_contents(old_contents)
         now = time.time()
-        for e in self.entries:
-            if len(e) == 2:
-                name, child = e
-                new_metadata = None
-            else:
-                assert len(e) == 3
-                name, child, new_metadata = e
-                assert _assert(IFilesystemNode.providedBy(child), child)
-            assert isinstance(name, unicode)
+        for (name, (child, new_metadata)) in self.entries.iteritems():
+            precondition(isinstance(name, unicode), name)
+            precondition(IFilesystemNode.providedBy(child), child)
+
+            # Strictly speaking this is redundant because we would raise the
+            # error again in pack_children.
+            child.raise_error()
+
             if name in children:
                 if not self.overwrite:
                     raise ExistingChildError("child '%s' already exists" % name)
@@ -143,71 +134,105 @@ class Adder:
         new_contents = self.node._pack_contents(children)
         return new_contents
 
+def _encrypt_rw_uri(filenode, rw_uri):
+    assert isinstance(rw_uri, str)
+    writekey = filenode.get_writekey()
+    if not writekey:
+        return ""
+    salt = hashutil.mutable_rwcap_salt_hash(rw_uri)
+    key = hashutil.mutable_rwcap_key_hash(salt, writekey)
+    cryptor = AES(key)
+    crypttext = cryptor.process(rw_uri)
+    mac = hashutil.hmac(key, salt + crypttext)
+    assert len(mac) == 32
+    return salt + crypttext + mac
+    # The MAC is not checked by readers in Tahoe >= 1.3.0, but we still
+    # produce it for the sake of older readers.
+
+def pack_children(filenode, children, deep_immutable=False):
+    """Take a dict that maps:
+         children[unicode_name] = (IFileSystemNode, metadata_dict)
+    and pack it into a single string, for use as the contents of the backing
+    file. This is the same format as is returned by _unpack_contents. I also
+    accept an AuxValueDict, in which case I'll use the auxilliary cached data
+    as the pre-packed entry, which is faster than re-packing everything each
+    time.
+
+    If deep_immutable is True, I will require that all my children are deeply
+    immutable, and will raise a MustBeDeepImmutableError if not.
+    """
+
+    has_aux = isinstance(children, AuxValueDict)
+    entries = []
+    for name in sorted(children.keys()):
+        assert isinstance(name, unicode)
+        entry = None
+        (child, metadata) = children[name]
+        child.raise_error()
+        if deep_immutable and not child.is_allowed_in_immutable_directory():
+            raise MustBeDeepImmutableError("child '%s' is not allowed in an immutable directory" % (name,), name)
+        if has_aux:
+            entry = children.get_aux(name)
+        if not entry:
+            assert IFilesystemNode.providedBy(child), (name,child)
+            assert isinstance(metadata, dict)
+            rw_uri = child.get_write_uri()
+            if rw_uri is None:
+                rw_uri = ""
+            assert isinstance(rw_uri, str), rw_uri
+            
+            # should be prevented by MustBeDeepImmutableError check above
+            assert not (rw_uri and deep_immutable)
+
+            ro_uri = child.get_readonly_uri()
+            if ro_uri is None:
+                ro_uri = ""
+            assert isinstance(ro_uri, str), ro_uri
+            entry = "".join([netstring(name.encode("utf-8")),
+                             netstring(strip_prefix_for_ro(ro_uri, deep_immutable)),
+                             netstring(_encrypt_rw_uri(filenode, rw_uri)),
+                             netstring(simplejson.dumps(metadata))])
+        entries.append(netstring(entry))
+    return "".join(entries)
+
 class DirectoryNode:
     implements(IDirectoryNode, ICheckable, IDeepCheckable)
     filenode_class = MutableFileNode
 
-    def __init__(self, client):
-        self._client = client
-        self._most_recent_size = None
+    def __init__(self, filenode, nodemaker, uploader):
+        assert IFileNode.providedBy(filenode), filenode
+        assert not IDirectoryNode.providedBy(filenode), filenode
+        self._node = filenode
+        filenode_cap = filenode.get_cap()
+        self._uri = wrap_dirnode_cap(filenode_cap)
+        self._nodemaker = nodemaker
+        self._uploader = uploader
 
     def __repr__(self):
-        return "<%s %s %s>" % (self.__class__.__name__, self.is_readonly() and "RO" or "RW", hasattr(self, '_uri') and self._uri.abbrev())
-    def init_from_uri(self, myuri):
-        self._uri = IURI(myuri)
-        self._node = self.filenode_class(self._client)
-        self._node.init_from_uri(self._uri.get_filenode_uri())
-        return self
-
-    @classmethod
-    def create_with_mutablefile(cls, filenode, client):
-        self = cls(client)
-        self._node = filenode
-        return self._filenode_created(filenode)
-
-    def create(self, keypair_generator=None, keysize=None):
-        """
-        Returns a deferred that eventually fires with self once the directory
-        has been created (distributed across a set of storage servers).
-        """
-        # first we create a MutableFileNode with empty_contents, then use its
-        # URI to create our own.
-        self._node = self.filenode_class(self._client)
-        empty_contents = self._pack_contents(CachingDict())
-        d = self._node.create(empty_contents, keypair_generator, keysize=keysize)
-        d.addCallback(self._filenode_created)
-        return d
-    def _filenode_created(self, res):
-        self._uri = DirectoryURI(IMutableFileURI(self._node.get_uri()))
-        return self
+        return "<%s %s-%s %s>" % (self.__class__.__name__,
+                                  self.is_readonly() and "RO" or "RW",
+                                  self.is_mutable() and "MUT" or "IMM",
+                                  hasattr(self, '_uri') and self._uri.abbrev())
 
     def get_size(self):
-        # return the size of our backing mutable file, in bytes, if we've
-        # fetched it.
-        return self._most_recent_size
+        """Return the size of our backing mutable file, in bytes, if we've
+        fetched it. Otherwise return None. This returns synchronously."""
+        return self._node.get_size()
 
-    def _set_size(self, data):
-        self._most_recent_size = len(data)
-        return data
+    def get_current_size(self):
+        """Calculate the size of our backing mutable file, in bytes. Returns
+        a Deferred that fires with the result."""
+        return self._node.get_current_size()
 
     def _read(self):
-        d = self._node.download_best_version()
-        d.addCallback(self._set_size)
+        if self._node.is_mutable():
+            # use the IMutableFileNode API.
+            d = self._node.download_best_version()
+        else:
+            d = download_to_data(self._node)
         d.addCallback(self._unpack_contents)
         return d
 
-    def _encrypt_rwcap(self, rwcap):
-        assert isinstance(rwcap, str)
-        salt = hashutil.mutable_rwcap_salt_hash(rwcap)
-        key = hashutil.mutable_rwcap_key_hash(salt, self._node.get_writekey())
-        cryptor = AES(key)
-        crypttext = cryptor.process(rwcap)
-        mac = hashutil.hmac(key, salt + crypttext)
-        assert len(mac) == 32
-        return salt + crypttext + mac
-        # The MAC is not checked by readers in Tahoe >= 1.3.0, but we still
-        # produce it for the sake of older readers.
-
     def _decrypt_rwcapdata(self, encwrcap):
         salt = encwrcap[:16]
         crypttext = encwrcap[16:-32]
@@ -216,87 +241,112 @@ class DirectoryNode:
         plaintext = cryptor.process(crypttext)
         return plaintext
 
-    def _create_node(self, rwcap, rocap):
-        return self._client.create_node_from_uri(rwcap, rocap)
+    def _create_and_validate_node(self, rw_uri, ro_uri, name):
+        node = self._nodemaker.create_from_cap(rw_uri, ro_uri,
+                                               deep_immutable=not self.is_mutable(),
+                                               name=name)
+        node.raise_error()
+        return node
 
     def _unpack_contents(self, data):
         # the directory is serialized as a list of netstrings, one per child.
-        # Each child is serialized as a list of four netstrings: (name,
-        # rocap, rwcap, metadata), in which the name,rocap,metadata are in
-        # cleartext. The 'name' is UTF-8 encoded. The rwcap is formatted as:
-        # pack("16ss32s", iv, AES(H(writekey+iv), plaintextrwcap), mac)
+        # Each child is serialized as a list of four netstrings: (name, ro_uri,
+        # rwcapdata, metadata), in which the name, ro_uri, metadata are in
+        # cleartext. The 'name' is UTF-8 encoded. The rwcapdata is formatted as:
+        # pack("16ss32s", iv, AES(H(writekey+iv), plaintext_rw_uri), mac)
         assert isinstance(data, str), (repr(data), type(data))
         # an empty directory is serialized as an empty string
         if data == "":
-            return CachingDict()
+            return AuxValueDict()
         writeable = not self.is_readonly()
-        children = CachingDict()
+        mutable = self.is_mutable()
+        children = AuxValueDict()
         position = 0
         while position < len(data):
             entries, position = split_netstring(data, 1, position)
             entry = entries[0]
-            (name, rocap, rwcapdata, metadata_s), subpos = split_netstring(entry, 4)
-            name = name.decode("utf-8")
-            rwcap = None
+            (name_utf8, ro_uri, rwcapdata, metadata_s), subpos = split_netstring(entry, 4)
+            if not mutable and len(rwcapdata) > 0:
+                raise ValueError("the rwcapdata field of a dirnode in an immutable directory was not empty")
+            name = name_utf8.decode("utf-8")
+            rw_uri = ""
             if writeable:
-                rwcap = self._decrypt_rwcapdata(rwcapdata)
-            if not rwcap:
-                rwcap = None # rwcap is None or a non-empty string
-            if not rocap:
-                rocap = None # rocap is None or a non-empty string
-            child = self._create_node(rwcap, rocap)
-            metadata = simplejson.loads(metadata_s)
-            assert isinstance(metadata, dict)
-            children.set_both_items(name, entry, (child, metadata))
+                rw_uri = self._decrypt_rwcapdata(rwcapdata)
+
+            # Since the encryption uses CTR mode, it currently leaks the length of the
+            # plaintext rw_uri -- and therefore whether it is present, i.e. whether the
+            # dirnode is writeable (ticket #925). By stripping trailing spaces in
+            # Tahoe >= 1.6.0, we may make it easier for future versions to plug this leak.
+            # ro_uri is treated in the same way for consistency.
+            # rw_uri and ro_uri will be either None or a non-empty string.
+
+            rw_uri = rw_uri.rstrip(' ') or None
+            ro_uri = ro_uri.rstrip(' ') or None
+
+            try:
+                child = self._create_and_validate_node(rw_uri, ro_uri, name)
+                if mutable or child.is_allowed_in_immutable_directory():
+                    metadata = simplejson.loads(metadata_s)
+                    assert isinstance(metadata, dict)
+                    children[name] = (child, metadata)
+                    children.set_with_aux(name, (child, metadata), auxilliary=entry)
+                else:
+                    log.msg(format="mutable cap for child '%(name)s' unpacked from an immutable directory",
+                                   name=name_utf8,
+                                   facility="tahoe.webish", level=log.UNUSUAL)
+            except CapConstraintError, e:
+                log.msg(format="unmet constraint on cap for child '%(name)s' unpacked from a directory:\n"
+                               "%(message)s", message=e.args[0], name=name_utf8,
+                               facility="tahoe.webish", level=log.UNUSUAL)
+
         return children
 
     def _pack_contents(self, children):
         # expects children in the same format as _unpack_contents
-        assert isinstance(children, CachingDict)
-        entries = []
-        for name in sorted(children.keys()):
-            entry, metadata = children.get_both_items(name)
-            if entry == None:
-                child, metadata = metadata
-                assert isinstance(name, unicode)
-                assert IFilesystemNode.providedBy(child), (name,child)
-                assert isinstance(metadata, dict)
-                rwcap = child.get_uri() # might be RO if the child is not writeable
-                if rwcap is None:
-                    rwcap = ""
-                assert isinstance(rwcap, str), rwcap
-                rocap = child.get_readonly_uri()
-                if rocap is None:
-                    rocap = ""
-                assert isinstance(rocap, str), rocap
-                entry = "".join([netstring(name.encode("utf-8")),
-                             netstring(rocap),
-                             netstring(self._encrypt_rwcap(rwcap)),
-                             netstring(simplejson.dumps(metadata))])
-            entries.append(netstring(entry))
-        return "".join(entries)
+        return pack_children(self._node, children)
 
     def is_readonly(self):
         return self._node.is_readonly()
+
     def is_mutable(self):
         return self._node.is_mutable()
 
+    def is_unknown(self):
+        return False
+
+    def is_allowed_in_immutable_directory(self):
+        return not self._node.is_mutable()
+
+    def raise_error(self):
+        pass
+
     def get_uri(self):
         return self._uri.to_string()
 
+    def get_write_uri(self):
+        if self.is_readonly():
+            return None
+        return self._uri.to_string()
+
     def get_readonly_uri(self):
         return self._uri.get_readonly().to_string()
 
+    def get_cap(self):
+        return self._uri
+
+    def get_readcap(self):
+        return self._uri.get_readonly()
+
     def get_verify_cap(self):
         return self._uri.get_verify_cap()
 
     def get_repair_cap(self):
         if self._node.is_readonly():
-            return None
+            return None # readonly (mutable) dirnodes are not yet repairable
         return self._uri
 
     def get_storage_index(self):
-        return self._uri._filenode_uri.storage_index
+        return self._uri.get_storage_index()
 
     def check(self, monitor, verify=False, add_lease=False):
         """Perform a file check. See IChecker.check for details."""
@@ -306,7 +356,7 @@ class DirectoryNode:
 
     def list(self):
         """I return a Deferred that fires with a dictionary mapping child
-        name to a tuple of (IFileNode or IDirectoryNode, metadata)."""
+        name to a tuple of (IFilesystemNode, metadata)."""
         return self._read()
 
     def has_child(self, name):
@@ -331,7 +381,7 @@ class DirectoryNode:
 
     def get(self, name):
         """I return a Deferred that fires with the named child node,
-        which is either an IFileNode or an IDirectoryNode."""
+        which is an IFilesystemNode."""
         assert isinstance(name, unicode)
         d = self._read()
         d.addCallback(self._get, name)
@@ -339,8 +389,8 @@ class DirectoryNode:
 
     def get_child_and_metadata(self, name):
         """I return a Deferred that fires with the (node, metadata) pair for
-        the named child. The node is either an IFileNode or an
-        IDirectoryNode, and the metadata is a dictionary."""
+        the named child. The node is an IFilesystemNode, and the metadata
+        is a dictionary."""
         assert isinstance(name, unicode)
         d = self._read()
         d.addCallback(self._get_with_metadata, name)
@@ -355,7 +405,7 @@ class DirectoryNode:
     def set_metadata_for(self, name, metadata):
         assert isinstance(name, unicode)
         if self.is_readonly():
-            return defer.fail(NotMutableError())
+            return defer.fail(NotWriteableError())
         assert isinstance(metadata, dict)
         s = MetadataSetter(self, name, metadata)
         d = self._node.modify(s.modify)
@@ -363,7 +413,7 @@ class DirectoryNode:
         return d
 
     def get_child_at_path(self, path):
-        """Transform a child path into an IDirectoryNode or IFileNode.
+        """Transform a child path into an IFilesystemNode.
 
         I perform a recursive series of 'get' operations to find the named
         descendant node. I return a Deferred that fires with the node, or
@@ -377,7 +427,7 @@ class DirectoryNode:
         return d
 
     def get_child_and_metadata_at_path(self, path):
-        """Transform a child path into an IDirectoryNode or IFileNode and
+        """Transform a child path into an IFilesystemNode and
         a metadata dictionary from the last edge that was traversed.
         """
 
@@ -399,26 +449,14 @@ class DirectoryNode:
         d = self.get_child_and_metadata(childname)
         return d
 
-    def set_uri(self, name, child_uri, metadata=None, overwrite=True):
-        """I add a child (by URI) at the specific name. I return a Deferred
-        that fires with the child node when the operation finishes. I will
-        replace any existing child of the same name.
-
-        The child_uri could be for a file, or for a directory (either
-        read-write or read-only, using a URI that came from get_uri() ).
-
-        If this directory node is read-only, the Deferred will errback with a
-        NotMutableError."""
+    def set_uri(self, name, writecap, readcap, metadata=None, overwrite=True):
         precondition(isinstance(name, unicode), name)
-        precondition(isinstance(child_uri, str), child_uri)
-        child_node = self._create_node(child_uri, None)
-        if isinstance(child_node, UnknownNode):
-            # don't be willing to pack unknown nodes: we might accidentally
-            # put some write-authority into the rocap slot because we don't
-            # know how to diminish the URI they gave us. We don't even know
-            # if they gave us a readcap or a writecap.
-            msg = "cannot pack unknown node as child %s" % str(name)
-            raise CannotPackUnknownNodeError(msg)
+        precondition(isinstance(writecap, (str,type(None))), writecap)
+        precondition(isinstance(readcap, (str,type(None))), readcap)
+            
+        # We now allow packing unknown nodes, provided they are valid
+        # for this type of directory.
+        child_node = self._create_and_validate_node(writecap, readcap, name)
         d = self.set_node(name, child_node, metadata, overwrite)
         d.addCallback(lambda res: child_node)
         return d
@@ -426,21 +464,24 @@ class DirectoryNode:
     def set_children(self, entries, overwrite=True):
         # this takes URIs
         a = Adder(self, overwrite=overwrite)
-        node_entries = []
-        for e in entries:
+        for (name, e) in entries.iteritems():
+            assert isinstance(name, unicode)
             if len(e) == 2:
-                name, child_uri = e
+                writecap, readcap = e
                 metadata = None
             else:
                 assert len(e) == 3
-                name, child_uri, metadata = e
-            assert isinstance(name, unicode)
-            child_node = self._create_node(child_uri, None)
-            if isinstance(child_node, UnknownNode):
-                msg = "cannot pack unknown node as child %s" % str(name)
-                raise CannotPackUnknownNodeError(msg)
+                writecap, readcap, metadata = e
+            precondition(isinstance(writecap, (str,type(None))), writecap)
+            precondition(isinstance(readcap, (str,type(None))), readcap)
+            
+            # We now allow packing unknown nodes, provided they are valid
+            # for this type of directory.
+            child_node = self._create_and_validate_node(writecap, readcap, name)
             a.set_node(name, child_node, metadata)
-        return self._node.modify(a.modify)
+        d = self._node.modify(a.modify)
+        d.addCallback(lambda ign: self)
+        return d
 
     def set_node(self, name, child, metadata=None, overwrite=True):
         """I add a child at the specific name. I return a Deferred that fires
@@ -449,12 +490,12 @@ class DirectoryNode:
         same name.
 
         If this directory node is read-only, the Deferred will errback with a
-        NotMutableError."""
+        NotWriteableError."""
 
         precondition(IFilesystemNode.providedBy(child), child)
 
         if self.is_readonly():
-            return defer.fail(NotMutableError())
+            return defer.fail(NotWriteableError())
         assert isinstance(name, unicode)
         assert IFilesystemNode.providedBy(child), child
         a = Adder(self, overwrite=overwrite)
@@ -464,11 +505,12 @@ class DirectoryNode:
         return d
 
     def set_nodes(self, entries, overwrite=True):
+        precondition(isinstance(entries, dict), entries)
         if self.is_readonly():
-            return defer.fail(NotMutableError())
+            return defer.fail(NotWriteableError())
         a = Adder(self, entries, overwrite=overwrite)
         d = self._node.modify(a.modify)
-        d.addCallback(lambda res: None)
+        d.addCallback(lambda res: self)
         return d
 
 
@@ -479,10 +521,10 @@ class DirectoryNode:
         the operation completes."""
         assert isinstance(name, unicode)
         if self.is_readonly():
-            return defer.fail(NotMutableError())
-        d = self._client.upload(uploadable)
-        d.addCallback(lambda results: results.uri)
-        d.addCallback(self._client.create_node_from_uri)
+            return defer.fail(NotWriteableError())
+        d = self._uploader.upload(uploadable)
+        d.addCallback(lambda results:
+                      self._create_and_validate_node(results.uri, None, name))
         d.addCallback(lambda node:
                       self.set_node(name, node, metadata, overwrite))
         return d
@@ -492,22 +534,23 @@ class DirectoryNode:
         fires (with the node just removed) when the operation finishes."""
         assert isinstance(name, unicode)
         if self.is_readonly():
-            return defer.fail(NotMutableError())
+            return defer.fail(NotWriteableError())
         deleter = Deleter(self, name)
         d = self._node.modify(deleter.modify)
         d.addCallback(lambda res: deleter.old_child)
         return d
 
-    def create_empty_directory(self, name, overwrite=True):
-        """I create and attach an empty directory at the given name. I return
-        a Deferred that fires (with the new directory node) when the
-        operation finishes."""
+    def create_subdirectory(self, name, initial_children={}, overwrite=True,
+                            mutable=True):
         assert isinstance(name, unicode)
         if self.is_readonly():
-            return defer.fail(NotMutableError())
-        d = self._client.create_empty_dirnode()
+            return defer.fail(NotWriteableError())
+        if mutable:
+            d = self._nodemaker.create_new_mutable_directory(initial_children)
+        else:
+            d = self._nodemaker.create_immutable_directory(initial_children)
         def _created(child):
-            entries = [(name, child, None)]
+            entries = {name: (child, None)}
             a = Adder(self, entries, overwrite=overwrite)
             d = self._node.modify(a.modify)
             d.addCallback(lambda res: child)
@@ -523,7 +566,7 @@ class DirectoryNode:
         Deferred that fires when the operation finishes."""
         assert isinstance(current_child_name, unicode)
         if self.is_readonly() or new_parent.is_readonly():
-            return defer.fail(NotMutableError())
+            return defer.fail(NotWriteableError())
         if new_child_name is None:
             new_child_name = current_child_name
         assert isinstance(new_child_name, unicode)
@@ -695,7 +738,7 @@ class DeepStats:
             self.add("count-mutable-files")
             # TODO: update the servermap, compute a size, add it to
             # size-mutable-files, max it into "largest-mutable-file"
-        elif IFileNode.providedBy(node): # CHK and LIT
+        elif IImmutableFileNode.providedBy(node): # CHK and LIT
             self.add("count-files")
             size = node.get_size()
             self.histogram("size-files-histogram", size)
@@ -710,9 +753,10 @@ class DeepStats:
 
     def enter_directory(self, parent, children):
         dirsize_bytes = parent.get_size()
+        if dirsize_bytes is not None:
+            self.add("size-directories", dirsize_bytes)
+            self.max("largest-directory", dirsize_bytes)
         dirsize_children = len(children)
-        self.add("size-directories", dirsize_bytes)
-        self.max("largest-directory", dirsize_bytes)
         self.max("largest-directory-children", dirsize_children)
 
     def add(self, key, value=1):
@@ -786,9 +830,13 @@ class ManifestWalker(DeepStats):
 class DeepChecker:
     def __init__(self, root, verify, repair, add_lease):
         root_si = root.get_storage_index()
+        if root_si:
+            root_si_base32 = base32.b2a(root_si)
+        else:
+            root_si_base32 = ""
         self._lp = log.msg(format="deep-check starting (%(si)s),"
                            " verify=%(verify)s, repair=%(repair)s",
-                           si=base32.b2a(root_si), verify=verify, repair=repair)
+                           si=root_si_base32, verify=verify, repair=repair)
         self._verify = verify
         self._repair = repair
         self._add_lease = add_lease
