@@ -4,13 +4,12 @@ import random
 from zope.interface import implements
 from twisted.internet import defer, reactor
 from foolscap.api import eventually
-from allmydata.interfaces import IMutableFileNode, IMutableFileURI, \
+from allmydata.interfaces import IMutableFileNode, \
      ICheckable, ICheckResults, NotEnoughSharesError
 from allmydata.util import hashutil, log
 from allmydata.util.assertutil import precondition
-from allmydata.uri import WriteableSSKFileURI
+from allmydata.uri import WriteableSSKFileURI, ReadonlySSKFileURI
 from allmydata.monitor import Monitor
-from pycryptopp.publickey import rsa
 from pycryptopp.cipher.aes import AES
 
 from publish import Publish
@@ -44,26 +43,27 @@ class BackoffAgent:
         reactor.callLater(self._delay, d.callback, None)
         return d
 
-# use client.create_mutable_file() to make one of these
+# use nodemaker.create_mutable_file() to make one of these
 
 class MutableFileNode:
     implements(IMutableFileNode, ICheckable)
-    SIGNATURE_KEY_SIZE = 2048
-    checker_class = MutableChecker
-    check_and_repairer_class = MutableCheckAndRepairer
 
-    def __init__(self, client):
-        self._client = client
+    def __init__(self, storage_broker, secret_holder,
+                 default_encoding_parameters, history):
+        self._storage_broker = storage_broker
+        self._secret_holder = secret_holder
+        self._default_encoding_parameters = default_encoding_parameters
+        self._history = history
         self._pubkey = None # filled in upon first read
         self._privkey = None # filled in if we're mutable
         # we keep track of the last encoding parameters that we use. These
         # are updated upon retrieve, and used by publish. If we publish
         # without ever reading (i.e. overwrite()), then we use these values.
-        defaults = client.get_encoding_parameters()
-        self._required_shares = defaults["k"]
-        self._total_shares = defaults["n"]
+        self._required_shares = default_encoding_parameters["k"]
+        self._total_shares = default_encoding_parameters["n"]
         self._sharemap = {} # known shares, shnum-to-[nodeids]
         self._cache = ResponseCache()
+        self._most_recent_size = None
 
         # all users of this MutableFileNode go through the serializer. This
         # takes advantage of the fact that Deferreds discard the callbacks
@@ -77,16 +77,16 @@ class MutableFileNode:
         else:
             return "<%s %x %s %s>" % (self.__class__.__name__, id(self), None, None)
 
-    def init_from_uri(self, myuri):
+    def init_from_cap(self, filecap):
         # we have the URI, but we have not yet retrieved the public
         # verification key, nor things like 'k' or 'N'. If and when someone
         # wants to get our contents, we'll pull from shares and fill those
         # in.
-        self._uri = IMutableFileURI(myuri)
-        if not self._uri.is_readonly():
+        assert isinstance(filecap, (ReadonlySSKFileURI, WriteableSSKFileURI))
+        self._uri = filecap
+        self._writekey = None
+        if isinstance(filecap, WriteableSSKFileURI):
             self._writekey = self._uri.writekey
-        else:
-            self._writekey = None
         self._readkey = self._uri.readkey
         self._storage_index = self._uri.storage_index
         self._fingerprint = self._uri.fingerprint
@@ -100,21 +100,13 @@ class MutableFileNode:
         self._encprivkey = None
         return self
 
-    def create(self, initial_contents, keypair_generator=None, keysize=None):
-        """Call this when the filenode is first created. This will generate
-        the keys, generate the initial shares, wait until at least numpeers
-        are connected, allocate shares, and upload the initial
-        contents. Returns a Deferred that fires (with the MutableFileNode
-        instance you should use) when it completes.
+    def create_with_keys(self, (pubkey, privkey), contents):
+        """Call this to create a brand-new mutable file. It will create the
+        shares, find homes for them, and upload the initial contents (created
+        with the same rules as IClient.create_mutable_file() ). Returns a
+        Deferred that fires (with the MutableFileNode instance you should
+        use) when it completes.
         """
-        keysize = keysize or self.SIGNATURE_KEY_SIZE
-        d = defer.maybeDeferred(self._generate_pubprivkeys,
-                                keypair_generator, keysize)
-        d.addCallback(self._generated)
-        d.addCallback(lambda res: self._upload(initial_contents, None))
-        return d
-
-    def _generated(self, (pubkey, privkey) ):
         self._pubkey, self._privkey = pubkey, privkey
         pubkey_s = self._pubkey.serialize()
         privkey_s = self._privkey.serialize()
@@ -124,16 +116,17 @@ class MutableFileNode:
         self._uri = WriteableSSKFileURI(self._writekey, self._fingerprint)
         self._readkey = self._uri.readkey
         self._storage_index = self._uri.storage_index
+        initial_contents = self._get_initial_contents(contents)
+        return self._upload(initial_contents, None)
 
-    def _generate_pubprivkeys(self, keypair_generator, keysize):
-        if keypair_generator:
-            return keypair_generator(keysize)
-        else:
-            # RSA key generation for a 2048 bit key takes between 0.8 and 3.2
-            # secs
-            signer = rsa.generate(keysize)
-            verifier = signer.get_verifying_key()
-            return verifier, signer
+    def _get_initial_contents(self, contents):
+        if isinstance(contents, str):
+            return contents
+        if contents is None:
+            return ""
+        assert callable(contents), "%s should be callable, not %s" % \
+               (contents, type(contents))
+        return contents(self)
 
     def _encrypt_privkey(self, writekey, privkey):
         enc = AES(writekey)
@@ -156,19 +149,22 @@ class MutableFileNode:
         self._privkey = privkey
     def _populate_encprivkey(self, encprivkey):
         self._encprivkey = encprivkey
-
+    def _add_to_cache(self, verinfo, shnum, offset, data, timestamp):
+        self._cache.add(verinfo, shnum, offset, data, timestamp)
+    def _read_from_cache(self, verinfo, shnum, offset, length):
+        return self._cache.read(verinfo, shnum, offset, length)
 
     def get_write_enabler(self, peerid):
         assert len(peerid) == 20
         return hashutil.ssk_write_enabler_hash(self._writekey, peerid)
     def get_renewal_secret(self, peerid):
         assert len(peerid) == 20
-        crs = self._client.get_renewal_secret()
+        crs = self._secret_holder.get_renewal_secret()
         frs = hashutil.file_renewal_secret_hash(crs, self._storage_index)
         return hashutil.bucket_renewal_secret_hash(frs, peerid)
     def get_cancel_secret(self, peerid):
         assert len(peerid) == 20
-        ccs = self._client.get_cancel_secret()
+        ccs = self._secret_holder.get_cancel_secret()
         fcs = hashutil.file_cancel_secret_hash(ccs, self._storage_index)
         return hashutil.bucket_cancel_secret_hash(fcs, peerid)
 
@@ -178,6 +174,8 @@ class MutableFileNode:
         return self._readkey
     def get_storage_index(self):
         return self._storage_index
+    def get_fingerprint(self):
+        return self._fingerprint
     def get_privkey(self):
         return self._privkey
     def get_encprivkey(self):
@@ -193,25 +191,61 @@ class MutableFileNode:
     ####################################
     # IFilesystemNode
 
+    def get_size(self):
+        return self._most_recent_size
+    def get_current_size(self):
+        d = self.get_size_of_best_version()
+        d.addCallback(self._stash_size)
+        return d
+    def _stash_size(self, size):
+        self._most_recent_size = size
+        return size
+
+    def get_cap(self):
+        return self._uri
+    def get_readcap(self):
+        return self._uri.get_readonly()
+    def get_verify_cap(self):
+        return self._uri.get_verify_cap()
+    def get_repair_cap(self):
+        if self._uri.is_readonly():
+            return None
+        return self._uri
+
     def get_uri(self):
         return self._uri.to_string()
-    def get_size(self):
-        return "?" # TODO: this is likely to cause problems, not being an int
-    def get_readonly(self):
+
+    def get_write_uri(self):
         if self.is_readonly():
-            return self
-        ro = MutableFileNode(self._client)
-        ro.init_from_uri(self._uri.get_readonly())
-        return ro
+            return None
+        return self._uri.to_string()
 
     def get_readonly_uri(self):
         return self._uri.get_readonly().to_string()
 
+    def get_readonly(self):
+        if self.is_readonly():
+            return self
+        ro = MutableFileNode(self._storage_broker, self._secret_holder,
+                             self._default_encoding_parameters, self._history)
+        ro.init_from_cap(self._uri.get_readonly())
+        return ro
+
     def is_mutable(self):
         return self._uri.is_mutable()
+
     def is_readonly(self):
         return self._uri.is_readonly()
 
+    def is_unknown(self):
+        return False
+
+    def is_allowed_in_immutable_directory(self):
+        return not self._uri.is_mutable()
+
+    def raise_error(self):
+        pass
+
     def __hash__(self):
         return hash((self.__class__, self._uri))
     def __cmp__(self, them):
@@ -221,14 +255,6 @@ class MutableFileNode:
             return cmp(self.__class__, them.__class__)
         return cmp(self._uri, them._uri)
 
-    def get_verify_cap(self):
-        return IMutableFileURI(self._uri).get_verify_cap()
-
-    def get_repair_cap(self):
-        if self._uri.is_readonly():
-            return None
-        return self._uri
-
     def _do_serialized(self, cb, *args, **kwargs):
         # note: to avoid deadlock, this callable is *not* allowed to invoke
         # other serialized methods within this (or any other)
@@ -251,11 +277,13 @@ class MutableFileNode:
     # ICheckable
 
     def check(self, monitor, verify=False, add_lease=False):
-        checker = self.checker_class(self, monitor)
+        checker = MutableChecker(self, self._storage_broker,
+                                 self._history, monitor)
         return checker.check(verify, add_lease)
 
     def check_and_repair(self, monitor, verify=False, add_lease=False):
-        checker = self.check_and_repairer_class(self, monitor)
+        checker = MutableCheckAndRepairer(self, self._storage_broker,
+                                          self._history, monitor)
         return checker.check(verify, add_lease)
 
     #################################
@@ -271,21 +299,6 @@ class MutableFileNode:
     #################################
     # IMutableFileNode
 
-    # allow the use of IDownloadTarget
-    def download(self, target):
-        # fake it. TODO: make this cleaner.
-        d = self.download_best_version()
-        def _done(data):
-            target.open(len(data))
-            target.write(data)
-            target.close()
-            return target.finish()
-        d.addCallback(_done)
-        return d
-
-
-    # new API
-
     def download_best_version(self):
         return self._do_serialized(self._download_best_version)
     def _download_best_version(self):
@@ -414,10 +427,10 @@ class MutableFileNode:
         servermap = ServerMap()
         return self._update_servermap(servermap, mode)
     def _update_servermap(self, servermap, mode):
-        u = ServermapUpdater(self, Monitor(), servermap, mode)
-        history = self._client.get_history()
-        if history:
-            history.notify_mapupdate(u.get_status())
+        u = ServermapUpdater(self, self._storage_broker, Monitor(), servermap,
+                             mode)
+        if self._history:
+            self._history.notify_mapupdate(u.get_status())
         return u.update()
 
     def download_version(self, servermap, version, fetch_privkey=False):
@@ -426,17 +439,25 @@ class MutableFileNode:
     def _try_once_to_download_version(self, servermap, version,
                                       fetch_privkey=False):
         r = Retrieve(self, servermap, version, fetch_privkey)
-        history = self._client.get_history()
-        if history:
-            history.notify_retrieve(r.get_status())
-        return r.download()
+        if self._history:
+            self._history.notify_retrieve(r.get_status())
+        d = r.download()
+        d.addCallback(self._downloaded_version)
+        return d
+    def _downloaded_version(self, data):
+        self._most_recent_size = len(data)
+        return data
 
     def upload(self, new_contents, servermap):
         return self._do_serialized(self._upload, new_contents, servermap)
     def _upload(self, new_contents, servermap):
         assert self._pubkey, "update_servermap must be called before publish"
-        p = Publish(self, servermap)
-        history = self._client.get_history()
-        if history:
-            history.notify_publish(p.get_status(), len(new_contents))
-        return p.publish(new_contents)
+        p = Publish(self, self._storage_broker, servermap)
+        if self._history:
+            self._history.notify_publish(p.get_status(), len(new_contents))
+        d = p.publish(new_contents)
+        d.addCallback(self._did_upload, len(new_contents))
+        return d
+    def _did_upload(self, res, size):
+        self._most_recent_size = size
+        return res
