@@ -10,7 +10,7 @@ from allmydata.util import base32, hashutil, mathutil, idlib, log
 from allmydata import hashtree, codec
 from allmydata.storage.server import si_b2a
 from pycryptopp.cipher.aes import AES
-from foolscap.api import eventually
+from foolscap.api import eventually, fireEventually
 
 from common import MODE_WRITE, MODE_CHECK, DictOfSets, \
      UncoordinatedWriteError, NotEnoughServersError
@@ -90,12 +90,13 @@ class Publish:
     To make the initial publish, set servermap to None.
     """
 
-    def __init__(self, filenode, servermap):
+    def __init__(self, filenode, storage_broker, servermap):
         self._node = filenode
+        self._storage_broker = storage_broker
         self._servermap = servermap
         self._storage_index = self._node.get_storage_index()
         self._log_prefix = prefix = si_b2a(self._storage_index)[:5]
-        num = self._node._client.log("Publish(%s): starting" % prefix)
+        num = self.log("Publish(%s): starting" % prefix, parent=None)
         self._log_number = num
         self._running = True
         self._first_write_error = None
@@ -176,7 +177,7 @@ class Publish:
         assert self._privkey
         self._encprivkey = self._node.get_encprivkey()
 
-        sb = self._node._client.get_storage_broker()
+        sb = self._storage_broker
         full_peerlist = sb.get_servers_for_index(self._storage_index)
         self.full_peerlist = full_peerlist # for use later, immutable
         self.bad_peers = set() # peerids who have errbacked/refused requests
@@ -373,7 +374,6 @@ class Publish:
                                         str(self._first_write_error),
                                         self._first_write_error)
 
-        new_assignments = []
         # we then index this peerlist with an integer, because we may have to
         # wrap. We update the goal as we go.
         i = 0
@@ -464,7 +464,7 @@ class Publish:
             # build the block hash tree. SDMF has only one leaf.
             leaves = [hashutil.block_hash(share_data)]
             t = hashtree.HashTree(leaves)
-            block_hash_trees[shnum] = block_hash_tree = list(t)
+            block_hash_trees[shnum] = list(t)
             share_hash_leaves[shnum] = t[0]
         for leaf in share_hash_leaves:
             assert leaf is not None
@@ -635,6 +635,8 @@ class Publish:
             d.addCallbacks(self._got_write_answer, self._got_write_error,
                            callbackArgs=(peerid, shnums, started),
                            errbackArgs=(peerid, shnums, started))
+            # tolerate immediate errback, like with DeadReferenceError
+            d.addBoth(fireEventually)
             d.addCallback(self.loop)
             d.addErrback(self._fatal_error)
 
@@ -819,7 +821,7 @@ class Publish:
             # TODO: recovery
         else:
             self.log("Publish done, success")
-            self._status.set_status("Done")
+            self._status.set_status("Finished")
             self._status.set_progress(1.0)
         eventually(self.done_deferred.callback, res)
 
