@@ -4,7 +4,8 @@ import urllib
 import simplejson
 from cStringIO import StringIO
 from twisted.python.failure import Failure
-from allmydata.scripts.common import get_alias, escape_path, DefaultAliasMarker
+from allmydata.scripts.common import get_alias, escape_path, \
+                                     DefaultAliasMarker, UnknownAliasError
 from allmydata.scripts.common_http import do_http
 from allmydata import uri
 
@@ -258,8 +259,7 @@ class TahoeDirectorySource:
                 readcap = ascii_or_none(data[1].get("ro_uri"))
                 self.children[name] = TahoeFileSource(self.nodeurl, mutable,
                                                       writecap, readcap)
-            else:
-                assert data[0] == "dirnode"
+            elif data[0] == "dirnode":
                 writecap = ascii_or_none(data[1].get("rw_uri"))
                 readcap = ascii_or_none(data[1].get("ro_uri"))
                 if writecap and writecap in self.cache:
@@ -277,6 +277,11 @@ class TahoeDirectorySource:
                     if recurse:
                         child.populate(True)
                 self.children[name] = child
+            else:
+                # TODO: there should be an option to skip unknown nodes.
+                raise TahoeError("Cannot copy unknown nodes (ticket #839). "
+                                 "You probably need to use a later version of "
+                                 "Tahoe-LAFS to copy this directory.")
 
 class TahoeMissingTarget:
     def __init__(self, url):
@@ -353,8 +358,7 @@ class TahoeDirectoryTarget:
                                                    urllib.quote(name.encode('utf-8'))])
                 self.children[name] = TahoeFileTarget(self.nodeurl, mutable,
                                                       writecap, readcap, url)
-            else:
-                assert data[0] == "dirnode"
+            elif data[0] == "dirnode":
                 writecap = ascii_or_none(data[1].get("rw_uri"))
                 readcap = ascii_or_none(data[1].get("ro_uri"))
                 if writecap and writecap in self.cache:
@@ -372,6 +376,11 @@ class TahoeDirectoryTarget:
                     if recurse:
                         child.populate(True)
                 self.children[name] = child
+            else:
+                # TODO: there should be an option to skip unknown nodes.
+                raise TahoeError("Cannot copy unknown nodes (ticket #839). "
+                                 "You probably need to use a later version of "
+                                 "Tahoe-LAFS to copy this directory.")
 
     def get_child_target(self, name):
         # return a new target for a named subdirectory of this dir
@@ -407,9 +416,11 @@ class TahoeDirectoryTarget:
         set_data = {}
         for (name, filecap) in self.new_children.items():
             # it just so happens that ?t=set_children will accept both file
-            # read-caps and write-caps as ['rw_uri'], and will handle eithe
+            # read-caps and write-caps as ['rw_uri'], and will handle either
             # correctly. So don't bother trying to figure out whether the one
             # we have is read-only or read-write.
+            # TODO: think about how this affects forward-compatibility for
+            # unknown caps
             set_data[name] = ["filenode", {"rw_uri": filecap}]
         body = simplejson.dumps(set_data)
         POST(url, body)
@@ -441,20 +452,24 @@ class Copier:
         self.caps_only = options["caps-only"]
         self.cache = {}
         try:
-            self.try_copy()
+            status = self.try_copy()
+            return status
         except TahoeError, te:
             Failure().printTraceback(self.stderr)
             print >>self.stderr
             te.display(self.stderr)
             return 1
-        return 0
 
     def try_copy(self):
         source_specs = self.options.sources
         destination_spec = self.options.destination
         recursive = self.options["recursive"]
 
-        target = self.get_target_info(destination_spec)
+        try:
+            target = self.get_target_info(destination_spec)
+        except UnknownAliasError, e:
+            self.to_stderr("error: %s" % e.args[0])
+            return 1
 
         try:
             sources = [] # list of (name, source object)
@@ -464,6 +479,9 @@ class Copier:
         except MissingSourceError, e:
             self.to_stderr("No such file or directory %s" % e.args[0])
             return 1
+        except UnknownAliasError, e:
+            self.to_stderr("error: %s" % e.args[0])
+            return 1
 
         have_source_dirs = bool([s for (name,s) in sources
                                  if isinstance(s, (LocalDirectorySource,
@@ -498,6 +516,13 @@ class Copier:
             return self.copy_file(source, target)
 
         if isinstance(target, (LocalDirectoryTarget, TahoeDirectoryTarget)):
+            # We're copying to an existing directory -- make sure that we 
+            # have target names for everything
+            for (name, source) in sources:
+                if name is None and isinstance(source, TahoeFileSource):
+                    self.to_stderr(
+                        "error: you must specify a destination filename")
+                    return 1
             return self.copy_to_directory(sources, target)
 
         self.to_stderr("unknown target")
@@ -523,7 +548,6 @@ class Copier:
             url = self.nodeurl + "uri/%s" % urllib.quote(rootcap)
             if path:
                 url += "/" + escape_path(path)
-                last_slash = path.rfind("/")
 
             resp = do_http("GET", url + "?t=json")
             if resp.status == 404:
@@ -587,6 +611,8 @@ class Copier:
                 writecap = ascii_or_none(d.get("rw_uri"))
                 readcap = ascii_or_none(d.get("ro_uri"))
                 mutable = d.get("mutable", False) # older nodes don't provide it
+                if source_spec.rfind('/') != -1:
+                    name = source_spec[source_spec.rfind('/')+1:]
                 t = TahoeFileSource(self.nodeurl, mutable, writecap, readcap)
         return name, t
 
@@ -762,6 +788,7 @@ def copy(options):
 #  local-file-in-the-way
 #   touch proposed
 #   tahoe cp -r my:docs/proposed/denver.txt proposed/denver.txt
+#  handling of unknown nodes
 
 # things that maybe should be errors but aren't
 #  local-dir-in-the-way
