@@ -1,14 +1,14 @@
 import os, random, struct
 from zope.interface import implements
 from twisted.internet import defer
-from twisted.internet.interfaces import IConsumer
+from twisted.internet.interfaces import IPullProducer
 from twisted.python import failure
 from twisted.application import service
 from twisted.web.error import Error as WebError
 from foolscap.api import flushEventualQueue, fireEventually
 from allmydata import uri, dirnode, client
 from allmydata.introducer.server import IntroducerNode
-from allmydata.interfaces import IURI, IMutableFileNode, IFileNode, \
+from allmydata.interfaces import IMutableFileNode, IImmutableFileNode, \
      FileTooLargeError, NotEnoughSharesError, ICheckable
 from allmydata.check_results import CheckResults, CheckAndRepairResults, \
      DeepCheckResults, DeepCheckAndRepairResults
@@ -18,6 +18,7 @@ from allmydata.storage.server import storage_index_to_dir
 from allmydata.storage.mutable import MutableShareFile
 from allmydata.util import hashutil, log, fileutil, pollmixin
 from allmydata.util.assertutil import precondition
+from allmydata.util.consumer import download_to_data
 from allmydata.stats import StatsGathererService
 from allmydata.key_generator import KeyGeneratorService
 import common_util as testutil
@@ -31,23 +32,31 @@ def flush_but_dont_ignore(res):
     d.addCallback(_done)
     return d
 
+class DummyProducer:
+    implements(IPullProducer)
+    def resumeProducing(self):
+        pass
+
 class FakeCHKFileNode:
-    """I provide IFileNode, but all of my data is stored in a class-level
-    dictionary."""
-    implements(IFileNode)
+    """I provide IImmutableFileNode, but all of my data is stored in a
+    class-level dictionary."""
+    implements(IImmutableFileNode)
     all_contents = {}
     bad_shares = {}
 
-    def __init__(self, u, thisclient):
-        precondition(IURI.providedBy(u), u)
-        self.client = thisclient
-        self.my_uri = u
-        self.storage_index = u.storage_index
+    def __init__(self, filecap):
+        precondition(isinstance(filecap, (uri.CHKFileURI, uri.LiteralFileURI)), filecap)
+        self.my_uri = filecap
+        self.storage_index = self.my_uri.get_storage_index()
 
     def get_uri(self):
         return self.my_uri.to_string()
+    def get_write_uri(self):
+        return None
     def get_readonly_uri(self):
         return self.my_uri.to_string()
+    def get_cap(self):
+        return self.my_uri
     def get_verify_cap(self):
         return self.my_uri.get_verify_cap()
     def get_repair_cap(self):
@@ -96,52 +105,61 @@ class FakeCHKFileNode:
         return False
     def is_readonly(self):
         return True
+    def is_unknown(self):
+        return False
+    def is_allowed_in_immutable_directory(self):
+        return True
+    def raise_error(self):
+        pass
 
-    def download(self, target):
-        if self.my_uri.to_string() not in self.all_contents:
-            f = failure.Failure(NotEnoughSharesError(None, 0, 3))
-            target.fail(f)
-            return defer.fail(f)
-        data = self.all_contents[self.my_uri.to_string()]
-        target.open(len(data))
-        target.write(data)
-        target.close()
-        return defer.maybeDeferred(target.finish)
-    def download_to_data(self):
-        if self.my_uri.to_string() not in self.all_contents:
-            return defer.fail(NotEnoughSharesError(None, 0, 3))
-        data = self.all_contents[self.my_uri.to_string()]
-        return defer.succeed(data)
     def get_size(self):
+        if isinstance(self.my_uri, uri.LiteralFileURI):
+            return self.my_uri.get_size()
         try:
             data = self.all_contents[self.my_uri.to_string()]
         except KeyError, le:
             raise NotEnoughSharesError(le, 0, 3)
         return len(data)
+
     def read(self, consumer, offset=0, size=None):
-        d = self.download_to_data()
-        def _got(data):
-            start = offset
-            if size is not None:
-                end = offset + size
-            else:
-                end = len(data)
-            consumer.write(data[start:end])
-            return consumer
-        d.addCallback(_got)
+        # we don't bother to call registerProducer/unregisterProducer,
+        # because it's a hassle to write a dummy Producer that does the right
+        # thing (we have to make sure that DummyProducer.resumeProducing
+        # writes the data into the consumer immediately, otherwise it will
+        # loop forever).
+
+        d = defer.succeed(None)
+        d.addCallback(self._read, consumer, offset, size)
         return d
 
-def make_chk_file_uri(size):
+    def _read(self, ignored, consumer, offset, size):
+        if isinstance(self.my_uri, uri.LiteralFileURI):
+            data = self.my_uri.data
+        else:
+            if self.my_uri.to_string() not in self.all_contents:
+                raise NotEnoughSharesError(None, 0, 3)
+            data = self.all_contents[self.my_uri.to_string()]
+        start = offset
+        if size is not None:
+            end = offset + size
+        else:
+            end = len(data)
+        consumer.write(data[start:end])
+        return consumer
+
+def make_chk_file_cap(size):
     return uri.CHKFileURI(key=os.urandom(16),
                           uri_extension_hash=os.urandom(32),
                           needed_shares=3,
                           total_shares=10,
                           size=size)
+def make_chk_file_uri(size):
+    return make_chk_file_cap(size).to_string()
 
-def create_chk_filenode(thisclient, contents):
-    u = make_chk_file_uri(len(contents))
-    n = FakeCHKFileNode(u, thisclient)
-    FakeCHKFileNode.all_contents[u.to_string()] = contents
+def create_chk_filenode(contents):
+    filecap = make_chk_file_cap(len(contents))
+    n = FakeCHKFileNode(filecap)
+    FakeCHKFileNode.all_contents[filecap.to_string()] = contents
     return n
 
 
@@ -154,23 +172,41 @@ class FakeMutableFileNode:
     all_contents = {}
     bad_shares = {}
 
-    def __init__(self, thisclient):
-        self.client = thisclient
-        self.my_uri = make_mutable_file_uri()
-        self.storage_index = self.my_uri.storage_index
-    def create(self, initial_contents, key_generator=None, keysize=None):
+    def __init__(self, storage_broker, secret_holder,
+                 default_encoding_parameters, history):
+        self.init_from_cap(make_mutable_file_cap())
+    def create(self, contents, key_generator=None, keysize=None):
+        initial_contents = self._get_initial_contents(contents)
         if len(initial_contents) > self.MUTABLE_SIZELIMIT:
             raise FileTooLargeError("SDMF is limited to one segment, and "
                                     "%d > %d" % (len(initial_contents),
                                                  self.MUTABLE_SIZELIMIT))
         self.all_contents[self.storage_index] = initial_contents
         return defer.succeed(self)
-    def init_from_uri(self, myuri):
-        self.my_uri = IURI(myuri)
-        self.storage_index = self.my_uri.storage_index
+    def _get_initial_contents(self, contents):
+        if isinstance(contents, str):
+            return contents
+        if contents is None:
+            return ""
+        assert callable(contents), "%s should be callable, not %s" % \
+               (contents, type(contents))
+        return contents(self)
+    def init_from_cap(self, filecap):
+        assert isinstance(filecap, (uri.WriteableSSKFileURI,
+                                    uri.ReadonlySSKFileURI))
+        self.my_uri = filecap
+        self.storage_index = self.my_uri.get_storage_index()
         return self
+    def get_cap(self):
+        return self.my_uri
+    def get_readcap(self):
+        return self.my_uri.get_readonly()
     def get_uri(self):
         return self.my_uri.to_string()
+    def get_write_uri(self):
+        if self.is_readonly():
+            return None
+        return self.my_uri.to_string()
     def get_readonly(self):
         return self.my_uri.get_readonly()
     def get_readonly_uri(self):
@@ -181,10 +217,18 @@ class FakeMutableFileNode:
         return self.my_uri.is_readonly()
     def is_mutable(self):
         return self.my_uri.is_mutable()
+    def is_unknown(self):
+        return False
+    def is_allowed_in_immutable_directory(self):
+        return not self.my_uri.is_mutable()
+    def raise_error(self):
+        pass
     def get_writekey(self):
         return "\x00"*16
     def get_size(self):
-        return "?" # TODO: see mutable.MutableFileNode.get_size
+        return len(self.all_contents[self.storage_index])
+    def get_current_size(self):
+        return self.get_size_of_best_version()
     def get_size_of_best_version(self):
         return defer.succeed(len(self.all_contents[self.storage_index]))
 
@@ -249,7 +293,12 @@ class FakeMutableFileNode:
         return d
 
     def download_best_version(self):
+        if isinstance(self.my_uri, uri.LiteralFileURI):
+            return defer.succeed(self.my_uri.data)
+        if self.storage_index not in self.all_contents:
+            return defer.fail(NotEnoughSharesError(None, 0, 3))
         return defer.succeed(self.all_contents[self.storage_index])
+
     def overwrite(self, new_contents):
         if len(new_contents) > self.MUTABLE_SIZELIMIT:
             raise FileTooLargeError("SDMF is limited to one segment, and "
@@ -267,28 +316,15 @@ class FakeMutableFileNode:
         self.all_contents[self.storage_index] = modifier(old_contents, None, True)
         return None
 
-    def download(self, target):
-        if self.storage_index not in self.all_contents:
-            f = failure.Failure(NotEnoughSharesError(None, 0, 3))
-            target.fail(f)
-            return defer.fail(f)
-        data = self.all_contents[self.storage_index]
-        target.open(len(data))
-        target.write(data)
-        target.close()
-        return defer.maybeDeferred(target.finish)
-    def download_to_data(self):
-        if self.storage_index not in self.all_contents:
-            return defer.fail(NotEnoughSharesError(None, 0, 3))
-        data = self.all_contents[self.storage_index]
-        return defer.succeed(data)
-
-def make_mutable_file_uri():
+def make_mutable_file_cap():
     return uri.WriteableSSKFileURI(writekey=os.urandom(16),
                                    fingerprint=os.urandom(32))
+def make_mutable_file_uri():
+    return make_mutable_file_cap().to_string()
+
 def make_verifier_uri():
     return uri.SSKVerifierURI(storage_index=os.urandom(16),
-                              fingerprint=os.urandom(32))
+                              fingerprint=os.urandom(32)).to_string()
 
 class FakeDirectoryNode(dirnode.DirectoryNode):
     """This offers IDirectoryNode, but uses a FakeMutableFileNode for the
@@ -436,7 +472,7 @@ class SystemTestMixin(pollmixin.PollMixin, testutil.StallMixin):
             if self.stats_gatherer_furl:
                 write("stats_gatherer.furl", self.stats_gatherer_furl)
 
-        # give subclasses a chance to append liens to the node's tahoe.cfg
+        # give subclasses a chance to append lines to the node's tahoe.cfg
         # files before they are launched.
         self._set_up_nodes_extra_config()
 
@@ -444,7 +480,7 @@ class SystemTestMixin(pollmixin.PollMixin, testutil.StallMixin):
         # will have registered the helper furl).
         c = self.add_service(client.Client(basedir=basedirs[0]))
         self.clients.append(c)
-        c.DEFAULT_MUTABLE_KEYSIZE = 522
+        c.set_default_mutable_keysize(522)
         d = c.when_tub_ready()
         def _ready(res):
             f = open(os.path.join(basedirs[0],"private","helper.furl"), "r")
@@ -460,7 +496,7 @@ class SystemTestMixin(pollmixin.PollMixin, testutil.StallMixin):
             for i in range(1, self.numclients):
                 c = self.add_service(client.Client(basedir=basedirs[i]))
                 self.clients.append(c)
-                c.DEFAULT_MUTABLE_KEYSIZE = 522
+                c.set_default_mutable_keysize(522)
             log.msg("STARTING")
             return self.wait_for_connections()
         d.addCallback(_ready)
@@ -497,7 +533,7 @@ class SystemTestMixin(pollmixin.PollMixin, testutil.StallMixin):
         def _stopped(res):
             new_c = client.Client(basedir=self.getdir("client%d" % num))
             self.clients[num] = new_c
-            new_c.DEFAULT_MUTABLE_KEYSIZE = 522
+            new_c.set_default_mutable_keysize(522)
             self.add_service(new_c)
             return new_c.when_tub_ready()
         d.addCallback(_stopped)
@@ -527,7 +563,7 @@ class SystemTestMixin(pollmixin.PollMixin, testutil.StallMixin):
 
         c = client.Client(basedir=basedir)
         self.clients.append(c)
-        c.DEFAULT_MUTABLE_KEYSIZE = 522
+        c.set_default_mutable_keysize(522)
         self.numclients += 1
         if add_to_sparent:
             c.setServiceParent(self.sparent)
@@ -904,8 +940,10 @@ class ShareManglingMixin(SystemTestMixin):
             cl0.DEFAULT_ENCODING_PARAMETERS['max_segment_size'] = 12
             d2 = cl0.upload(immutable.upload.Data(TEST_DATA, convergence=""))
             def _after_upload(u):
-                self.uri = IURI(u.uri)
-                return cl0.create_node_from_uri(self.uri)
+                filecap = u.uri
+                self.n = self.clients[1].create_node_from_uri(filecap)
+                self.uri = uri.CHKFileURI.init_from_string(filecap)
+                return cl0.create_node_from_uri(filecap)
             d2.addCallback(_after_upload)
             return d2
         d.addCallback(_upload_a_file)
@@ -972,7 +1010,7 @@ class ShareManglingMixin(SystemTestMixin):
         else:
             k = random.choice(ks)
         del shares[k]
-        self.replace_shares(shares, storage_index=self.uri.storage_index)
+        self.replace_shares(shares, storage_index=self.uri.get_storage_index())
 
         return unused
 
@@ -982,7 +1020,7 @@ class ShareManglingMixin(SystemTestMixin):
         assert ks, (shares.keys(), sharenum)
         k = ks[0]
         shares[k] = corruptor_func(shares[k])
-        self.replace_shares(shares, storage_index=self.uri.storage_index)
+        self.replace_shares(shares, storage_index=self.uri.get_storage_index())
         return corruptor_func
 
     def _corrupt_all_shares(self, unused, corruptor_func):
@@ -1022,9 +1060,7 @@ class ShareManglingMixin(SystemTestMixin):
         return sum_of_write_counts
 
     def _download_and_check_plaintext(self, unused=None):
-        self.downloader = self.clients[1].getServiceNamed("downloader")
-        d = self.downloader.download_to_data(self.uri)
-
+        d = download_to_data(self.n)
         def _after_download(result):
             self.failUnlessEqual(result, TEST_DATA)
         d.addCallback(_after_download)
@@ -1118,28 +1154,6 @@ class ErrorMixin(WebErrorMixin):
             print "First Error:", f.value.subFailure
         return f
 
-class MemoryConsumer:
-    implements(IConsumer)
-    def __init__(self):
-        self.chunks = []
-        self.done = False
-    def registerProducer(self, p, streaming):
-        if streaming:
-            # call resumeProducing once to start things off
-            p.resumeProducing()
-        else:
-            while not self.done:
-                p.resumeProducing()
-    def write(self, data):
-        self.chunks.append(data)
-    def unregisterProducer(self):
-        self.done = True
-
-def download_to_data(n, offset=0, size=None):
-    d = n.read(MemoryConsumer(), offset, size)
-    d.addCallback(lambda mc: "".join(mc.chunks))
-    return d
-
 def corrupt_field(data, offset, size, debug=False):
     if random.random() < 0.5:
         newdata = testutil.flip_one_bit(data, offset, size)
@@ -1152,23 +1166,23 @@ def corrupt_field(data, offset, size, debug=False):
             log.msg("testing: corrupting offset %d, size %d randomizing field, orig: %r, newval: %r" % (offset, size, data[offset:offset+size], newval))
         return data[:offset]+newval+data[offset+size:]
 
-def _corrupt_nothing(data):
-    """ Leave the data pristine. """
+def _corrupt_nothing(data, debug=False):
+    """Leave the data pristine. """
     return data
 
-def _corrupt_file_version_number(data):
-    """ Scramble the file data -- the share file version number have one bit flipped or else
-    will be changed to a random value."""
+def _corrupt_file_version_number(data, debug=False):
+    """Scramble the file data -- the share file version number have one bit
+    flipped or else will be changed to a random value."""
     return corrupt_field(data, 0x00, 4)
 
-def _corrupt_size_of_file_data(data):
-    """ Scramble the file data -- the field showing the size of the share data within the file
-    will be set to one smaller. """
+def _corrupt_size_of_file_data(data, debug=False):
+    """Scramble the file data -- the field showing the size of the share data
+    within the file will be set to one smaller."""
     return corrupt_field(data, 0x04, 4)
 
-def _corrupt_sharedata_version_number(data):
-    """ Scramble the file data -- the share data version number will have one bit flipped or
-    else will be changed to a random value, but not 1 or 2."""
+def _corrupt_sharedata_version_number(data, debug=False):
+    """Scramble the file data -- the share data version number will have one
+    bit flipped or else will be changed to a random value, but not 1 or 2."""
     return corrupt_field(data, 0x0c, 4)
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
@@ -1178,9 +1192,9 @@ def _corrupt_sharedata_version_number(data):
     newsharevernumbytes = struct.pack(">L", newsharevernum)
     return data[:0x0c] + newsharevernumbytes + data[0x0c+4:]
 
-def _corrupt_sharedata_version_number_to_plausible_version(data):
-    """ Scramble the file data -- the share data version number will
-    be changed to 2 if it is 1 or else to 1 if it is 2."""
+def _corrupt_sharedata_version_number_to_plausible_version(data, debug=False):
+    """Scramble the file data -- the share data version number will be
+    changed to 2 if it is 1 or else to 1 if it is 2."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1190,9 +1204,9 @@ def _corrupt_sharedata_version_number_to_plausible_version(data):
     newsharevernumbytes = struct.pack(">L", newsharevernum)
     return data[:0x0c] + newsharevernumbytes + data[0x0c+4:]
 
-def _corrupt_segment_size(data):
-    """ Scramble the file data -- the field showing the size of the segment will have one
-    bit flipped or else be changed to a random value. """
+def _corrupt_segment_size(data, debug=False):
+    """Scramble the file data -- the field showing the size of the segment
+    will have one bit flipped or else be changed to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1200,9 +1214,10 @@ def _corrupt_segment_size(data):
     else:
         return corrupt_field(data, 0x0c+0x04, 8, debug=False)
 
-def _corrupt_size_of_sharedata(data):
-    """ Scramble the file data -- the field showing the size of the data within the share
-    data will have one bit flipped or else will be changed to a random value. """
+def _corrupt_size_of_sharedata(data, debug=False):
+    """Scramble the file data -- the field showing the size of the data
+    within the share data will have one bit flipped or else will be changed
+    to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1210,9 +1225,10 @@ def _corrupt_size_of_sharedata(data):
     else:
         return corrupt_field(data, 0x0c+0x0c, 8)
 
-def _corrupt_offset_of_sharedata(data):
-    """ Scramble the file data -- the field showing the offset of the data within the share
-    data will have one bit flipped or else be changed to a random value. """
+def _corrupt_offset_of_sharedata(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the data
+    within the share data will have one bit flipped or else be changed to a
+    random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1220,9 +1236,10 @@ def _corrupt_offset_of_sharedata(data):
     else:
         return corrupt_field(data, 0x0c+0x14, 8)
 
-def _corrupt_offset_of_ciphertext_hash_tree(data):
-    """ Scramble the file data -- the field showing the offset of the ciphertext hash tree
-    within the share data will have one bit flipped or else be changed to a random value.
+def _corrupt_offset_of_ciphertext_hash_tree(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the
+    ciphertext hash tree within the share data will have one bit flipped or
+    else be changed to a random value.
     """
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
@@ -1231,9 +1248,10 @@ def _corrupt_offset_of_ciphertext_hash_tree(data):
     else:
         return corrupt_field(data, 0x0c+0x24, 8, debug=False)
 
-def _corrupt_offset_of_block_hashes(data):
-    """ Scramble the file data -- the field showing the offset of the block hash tree within
-    the share data will have one bit flipped or else will be changed to a random value. """
+def _corrupt_offset_of_block_hashes(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the block
+    hash tree within the share data will have one bit flipped or else will be
+    changed to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1241,10 +1259,11 @@ def _corrupt_offset_of_block_hashes(data):
     else:
         return corrupt_field(data, 0x0c+0x2c, 8)
 
-def _corrupt_offset_of_block_hashes_to_truncate_crypttext_hashes(data):
-    """ Scramble the file data -- the field showing the offset of the block hash tree within the
-    share data will have a multiple of hash size subtracted from it, thus causing the downloader
-    to download an incomplete crypttext hash tree."""
+def _corrupt_offset_of_block_hashes_to_truncate_crypttext_hashes(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the block
+    hash tree within the share data will have a multiple of hash size
+    subtracted from it, thus causing the downloader to download an incomplete
+    crypttext hash tree."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1258,9 +1277,10 @@ def _corrupt_offset_of_block_hashes_to_truncate_crypttext_hashes(data):
         newvalstr = struct.pack(">Q", newval)
         return data[:0x0c+0x2c]+newvalstr+data[0x0c+0x2c+8:]
 
-def _corrupt_offset_of_share_hashes(data):
-    """ Scramble the file data -- the field showing the offset of the share hash tree within
-    the share data will have one bit flipped or else will be changed to a random value. """
+def _corrupt_offset_of_share_hashes(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the share
+    hash tree within the share data will have one bit flipped or else will be
+    changed to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1268,9 +1288,10 @@ def _corrupt_offset_of_share_hashes(data):
     else:
         return corrupt_field(data, 0x0c+0x34, 8)
 
-def _corrupt_offset_of_uri_extension(data):
-    """ Scramble the file data -- the field showing the offset of the uri extension will
-    have one bit flipped or else will be changed to a random value. """
+def _corrupt_offset_of_uri_extension(data, debug=False):
+    """Scramble the file data -- the field showing the offset of the uri
+    extension will have one bit flipped or else will be changed to a random
+    value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1279,13 +1300,15 @@ def _corrupt_offset_of_uri_extension(data):
         return corrupt_field(data, 0x0c+0x3c, 8)
 
 def _corrupt_offset_of_uri_extension_to_force_short_read(data, debug=False):
-    """ Scramble the file data -- the field showing the offset of the uri extension will be set
-    to the size of the file minus 3.  This means when the client tries to read the length field
-    from that location it will get a short read -- the result string will be only 3 bytes long,
-    not the 4 or 8 bytes necessary to do a successful struct.unpack."""
+    """Scramble the file data -- the field showing the offset of the uri
+    extension will be set to the size of the file minus 3. This means when
+    the client tries to read the length field from that location it will get
+    a short read -- the result string will be only 3 bytes long, not the 4 or
+    8 bytes necessary to do a successful struct.unpack."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
-    # The "-0x0c" in here is to skip the server-side header in the share file, which the client doesn't see when seeking and reading.
+    # The "-0x0c" in here is to skip the server-side header in the share
+    # file, which the client doesn't see when seeking and reading.
     if sharevernum == 1:
         if debug:
             log.msg("testing: corrupting offset %d, size %d, changing %d to %d (len(data) == %d)" % (0x2c, 4, struct.unpack(">L", data[0x2c:0x2c+4])[0], len(data)-0x0c-3, len(data)))
@@ -1295,7 +1318,7 @@ def _corrupt_offset_of_uri_extension_to_force_short_read(data, debug=False):
             log.msg("testing: corrupting offset %d, size %d, changing %d to %d (len(data) == %d)" % (0x48, 8, struct.unpack(">Q", data[0x48:0x48+8])[0], len(data)-0x0c-3, len(data)))
         return data[:0x48] + struct.pack(">Q", len(data)-0x0c-3) + data[0x48+8:]
 
-def _corrupt_mutable_share_data(data):
+def _corrupt_mutable_share_data(data, debug=False):
     prefix = data[:32]
     assert prefix == MutableShareFile.MAGIC, "This function is designed to corrupt mutable shares of v1, and the magic number doesn't look right: %r vs %r" % (prefix, MutableShareFile.MAGIC)
     data_offset = MutableShareFile.DATA_OFFSET
@@ -1308,9 +1331,9 @@ def _corrupt_mutable_share_data(data):
     length = data_offset + offsets["enc_privkey"] - start
     return corrupt_field(data, start, length)
 
-def _corrupt_share_data(data):
-    """ Scramble the file data -- the field containing the share data itself will have one
-    bit flipped or else will be changed to a random value. """
+def _corrupt_share_data(data, debug=False):
+    """Scramble the file data -- the field containing the share data itself
+    will have one bit flipped or else will be changed to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways, not v%d." % sharevernum
     if sharevernum == 1:
@@ -1322,9 +1345,9 @@ def _corrupt_share_data(data):
 
         return corrupt_field(data, 0x0c+0x44, sharedatasize)
 
-def _corrupt_crypttext_hash_tree(data):
-    """ Scramble the file data -- the field containing the crypttext hash tree will have one
-    bit flipped or else will be changed to a random value.
+def _corrupt_crypttext_hash_tree(data, debug=False):
+    """Scramble the file data -- the field containing the crypttext hash tree
+    will have one bit flipped or else will be changed to a random value.
     """
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
@@ -1335,11 +1358,21 @@ def _corrupt_crypttext_hash_tree(data):
         crypttexthashtreeoffset = struct.unpack(">Q", data[0x0c+0x24:0x0c+0x24+8])[0]
         blockhashesoffset = struct.unpack(">Q", data[0x0c+0x2c:0x0c+0x2c+8])[0]
 
-    return corrupt_field(data, crypttexthashtreeoffset, blockhashesoffset-crypttexthashtreeoffset)
+    return corrupt_field(data, 0x0c+crypttexthashtreeoffset, blockhashesoffset-crypttexthashtreeoffset, debug=debug)
 
-def _corrupt_block_hashes(data):
-    """ Scramble the file data -- the field containing the block hash tree will have one bit
-    flipped or else will be changed to a random value.
+def _corrupt_crypttext_hash_tree_byte_x221(data, debug=False):
+    """Scramble the file data -- the byte at offset 0x221 will have its 7th
+    (b1) bit flipped.
+    """
+    sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
+    assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
+    if debug:
+        log.msg("original data: %r" % (data,))
+    return data[:0x0c+0x221] + chr(ord(data[0x0c+0x221])^0x02) + data[0x0c+0x2210+1:]
+
+def _corrupt_block_hashes(data, debug=False):
+    """Scramble the file data -- the field containing the block hash tree
+    will have one bit flipped or else will be changed to a random value.
     """
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
@@ -1350,11 +1383,11 @@ def _corrupt_block_hashes(data):
         blockhashesoffset = struct.unpack(">Q", data[0x0c+0x2c:0x0c+0x2c+8])[0]
         sharehashesoffset = struct.unpack(">Q", data[0x0c+0x34:0x0c+0x34+8])[0]
 
-    return corrupt_field(data, blockhashesoffset, sharehashesoffset-blockhashesoffset)
+    return corrupt_field(data, 0x0c+blockhashesoffset, sharehashesoffset-blockhashesoffset)
 
-def _corrupt_share_hashes(data):
-    """ Scramble the file data -- the field containing the share hash chain will have one
-    bit flipped or else will be changed to a random value.
+def _corrupt_share_hashes(data, debug=False):
+    """Scramble the file data -- the field containing the share hash chain
+    will have one bit flipped or else will be changed to a random value.
     """
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
@@ -1365,11 +1398,12 @@ def _corrupt_share_hashes(data):
         sharehashesoffset = struct.unpack(">Q", data[0x0c+0x34:0x0c+0x34+8])[0]
         uriextoffset = struct.unpack(">Q", data[0x0c+0x3c:0x0c+0x3c+8])[0]
 
-    return corrupt_field(data, sharehashesoffset, uriextoffset-sharehashesoffset)
+    return corrupt_field(data, 0x0c+sharehashesoffset, uriextoffset-sharehashesoffset)
 
-def _corrupt_length_of_uri_extension(data):
-    """ Scramble the file data -- the field showing the length of the uri extension will
-    have one bit flipped or else will be changed to a random value. """
+def _corrupt_length_of_uri_extension(data, debug=False):
+    """Scramble the file data -- the field showing the length of the uri
+    extension will have one bit flipped or else will be changed to a random
+    value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1377,11 +1411,11 @@ def _corrupt_length_of_uri_extension(data):
         return corrupt_field(data, uriextoffset, 4)
     else:
         uriextoffset = struct.unpack(">Q", data[0x0c+0x3c:0x0c+0x3c+8])[0]
-        return corrupt_field(data, uriextoffset, 8)
+        return corrupt_field(data, 0x0c+uriextoffset, 8)
 
-def _corrupt_uri_extension(data):
-    """ Scramble the file data -- the field containing the uri extension will have one bit
-    flipped or else will be changed to a random value. """
+def _corrupt_uri_extension(data, debug=False):
+    """Scramble the file data -- the field containing the uri extension will
+    have one bit flipped or else will be changed to a random value."""
     sharevernum = struct.unpack(">L", data[0x0c:0x0c+4])[0]
     assert sharevernum in (1, 2), "This test is designed to corrupt immutable shares of v1 or v2 in specific ways."
     if sharevernum == 1:
@@ -1391,4 +1425,4 @@ def _corrupt_uri_extension(data):
         uriextoffset = struct.unpack(">Q", data[0x0c+0x3c:0x0c+0x3c+8])[0]
         uriextlen = struct.unpack(">Q", data[0x0c+uriextoffset:0x0c+uriextoffset+8])[0]
 
-    return corrupt_field(data, uriextoffset, uriextlen)
+    return corrupt_field(data, 0x0c+uriextoffset, uriextlen)
