@@ -14,10 +14,9 @@
 # or the control.furl .
 
 import os.path
-import sha
 from zope.interface import implements
 from twisted.application import service
-from twisted.internet import reactor
+from twisted.internet import defer, reactor
 from twisted.python.failure import Failure
 from foolscap.api import Referenceable, fireEventually, RemoteException
 from base64 import b32encode
@@ -25,6 +24,7 @@ from allmydata import uri as tahoe_uri
 from allmydata.client import Client
 from allmydata.storage.server import StorageServer, storage_index_to_dir
 from allmydata.util import fileutil, idlib, hashutil
+from allmydata.util.hashutil import sha1
 from allmydata.test.common_web import HTTPClientGETFactory
 from allmydata.interfaces import IStorageBroker
 
@@ -38,11 +38,13 @@ class LocalWrapper:
     def __init__(self, original):
         self.original = original
         self.broken = False
+        self.hung_until = None
         self.post_call_notifier = None
         self.disconnectors = {}
 
     def callRemoteOnly(self, methname, *args, **kwargs):
         d = self.callRemote(methname, *args, **kwargs)
+        del d # explicitly ignored
         return None
 
     def callRemote(self, methname, *args, **kwargs):
@@ -56,11 +58,25 @@ class LocalWrapper:
                 return a
         args = tuple([wrap(a) for a in args])
         kwargs = dict([(k,wrap(kwargs[k])) for k in kwargs])
+
+        def _really_call():
+            meth = getattr(self.original, "remote_" + methname)
+            return meth(*args, **kwargs)
+
         def _call():
             if self.broken:
                 raise IntentionalError("I was asked to break")
-            meth = getattr(self.original, "remote_" + methname)
-            return meth(*args, **kwargs)
+            if self.hung_until:
+                d2 = defer.Deferred()
+                self.hung_until.addCallback(lambda ign: _really_call())
+                self.hung_until.addCallback(lambda res: d2.callback(res))
+                def _err(res):
+                    d2.errback(res)
+                    return res
+                self.hung_until.addErrback(_err)
+                return d2
+            return _really_call()
+
         d = fireEventually()
         d.addCallback(lambda res: _call())
         def _wrap_exception(f):
@@ -82,7 +98,7 @@ class LocalWrapper:
             return res
         d.addCallback(_return_membrane)
         if self.post_call_notifier:
-            d.addCallback(self.post_call_notifier, methname)
+            d.addCallback(self.post_call_notifier, self, methname)
         return d
 
     def notifyOnDisconnect(self, f, *args, **kwargs):
@@ -105,15 +121,13 @@ class NoNetworkStorageBroker:
     implements(IStorageBroker)
     def get_servers_for_index(self, key):
         return sorted(self.client._servers,
-                      key=lambda x: sha.new(key+x[0]).digest())
+                      key=lambda x: sha1(key+x[0]).digest())
     def get_all_servers(self):
         return frozenset(self.client._servers)
     def get_nickname_for_serverid(self, serverid):
         return None
 
 class NoNetworkClient(Client):
-    DEFAULT_MUTABLE_KEYSIZE = 522
-
     def create_tub(self):
         pass
     def init_introducer_client(self):
@@ -174,6 +188,7 @@ class NoNetworkGrid(service.MultiService):
         for i in range(num_servers):
             ss = self.make_server(i)
             self.add_server(i, ss)
+        self.rebuild_serverlist()
 
         for i in range(num_clients):
             clientid = hashutil.tagged_hash("clientid", str(i))[:20]
@@ -194,6 +209,7 @@ class NoNetworkGrid(service.MultiService):
                 c = client_config_hooks[i](clientdir)
             if not c:
                 c = NoNetworkClient(clientdir)
+                c.set_default_mutable_keysize(522)
             c.nodeid = clientid
             c.short_nodeid = b32encode(clientid).lower()[:8]
             c._servers = self.all_servers # can be updated later
@@ -217,10 +233,42 @@ class NoNetworkGrid(service.MultiService):
         serverid = ss.my_nodeid
         self.servers_by_number[i] = ss
         self.servers_by_id[serverid] = wrap_storage_server(ss)
+        self.rebuild_serverlist()
+
+    def rebuild_serverlist(self):
         self.all_servers = frozenset(self.servers_by_id.items())
         for c in self.clients:
             c._servers = self.all_servers
 
+    def remove_server(self, serverid):
+        # it's enough to remove the server from c._servers (we don't actually
+        # have to detach and stopService it)
+        for i,ss in self.servers_by_number.items():
+            if ss.my_nodeid == serverid:
+                del self.servers_by_number[i]
+                break
+        del self.servers_by_id[serverid]
+        self.rebuild_serverlist()
+
+    def break_server(self, serverid):
+        # mark the given server as broken, so it will throw exceptions when
+        # asked to hold a share or serve a share
+        self.servers_by_id[serverid].broken = True
+
+    def hang_server(self, serverid):
+        # hang the given server
+        ss = self.servers_by_id[serverid]
+        assert ss.hung_until is None
+        ss.hung_until = defer.Deferred()
+
+    def unhang_server(self, serverid):
+        # unhang the given server
+        ss = self.servers_by_id[serverid]
+        assert ss.hung_until is not None
+        ss.hung_until.callback(None)
+        ss.hung_until = None
+
+
 class GridTestMixin:
     def setUp(self):
         self.s = service.MultiService()
@@ -283,11 +331,11 @@ class GridTestMixin:
         corruptdata = corruptor_function(sharedata)
         open(sharefile, "wb").write(corruptdata)
 
-    def corrupt_shares_numbered(self, uri, shnums, corruptor):
+    def corrupt_shares_numbered(self, uri, shnums, corruptor, debug=False):
         for (i_shnum, i_serverid, i_sharefile) in self.find_shares(uri):
             if i_shnum in shnums:
                 sharedata = open(i_sharefile, "rb").read()
-                corruptdata = corruptor(sharedata)
+                corruptdata = corruptor(sharedata, debug=debug)
                 open(i_sharefile, "wb").write(corruptdata)
 
     def GET(self, urlpath, followRedirect=False, return_response=False,
