@@ -1,8 +1,10 @@
+# -*- coding: utf-8 -*-
 from allmydata.test import common
 from allmydata.monitor import Monitor
 from allmydata import check_results
 from allmydata.interfaces import NotEnoughSharesError
 from allmydata.immutable import repairer, upload
+from allmydata.util.consumer import download_to_data
 from twisted.internet import defer
 from twisted.trial import unittest
 import random
@@ -13,6 +15,7 @@ from no_network import GridTestMixin
 READ_LEEWAY = 18
 MAX_DELTA_READS = 10 * READ_LEEWAY # N = 10
 
+timeout=240 # François's ARM box timed out after 120 seconds of Verifier.test_corrupt_crypttext_hashtree
 
 class RepairTestMixin:
     def failUnlessIsInstance(self, x, xtype):
@@ -98,13 +101,13 @@ class Verifier(GridTestMixin, unittest.TestCase, RepairTestMixin):
         d.addCallback(_check2)
         return d
 
-    def _help_test_verify(self, corruptor, judgement, shnum=0):
+    def _help_test_verify(self, corruptor, judgement, shnum=0, debug=False):
         self.set_up_grid(num_clients=2)
         d = self.upload_and_stash()
         d.addCallback(lambda ignored: self._stash_counts())
 
         d.addCallback(lambda ignored:
-                      self.corrupt_shares_numbered(self.uri, [shnum],corruptor))
+                      self.corrupt_shares_numbered(self.uri, [shnum],corruptor,debug=debug))
         d.addCallback(lambda ignored:
                       self.c1_filenode.check(Monitor(), verify=True))
         def _check(vr):
@@ -134,23 +137,23 @@ class Verifier(GridTestMixin, unittest.TestCase, RepairTestMixin):
         self.failUnless(len(data['servers-responding']) == 10, data)
         self.failUnless(len(data['list-corrupt-shares']) == 0, data)
 
-    def test_verify_no_problem_1(self):
-        self.basedir = "repairer/Verify/verify_no_problem_1"
+    def test_ok_no_corruption(self):
+        self.basedir = "repairer/Verifier/ok_no_corruption"
         return self._help_test_verify(common._corrupt_nothing,
                                       self.judge_no_problem)
 
-    def test_verify_no_problem_2(self):
-        self.basedir = "repairer/Verify/verify_no_problem_2"
+    def test_ok_filedata_size(self):
+        self.basedir = "repairer/Verifier/ok_filedatasize"
         return self._help_test_verify(common._corrupt_size_of_file_data,
                                       self.judge_no_problem)
 
-    def test_verify_no_problem_3(self):
-        self.basedir = "repairer/Verify/verify_no_problem_3"
+    def test_ok_sharedata_size(self):
+        self.basedir = "repairer/Verifier/ok_sharedata_size"
         return self._help_test_verify(common._corrupt_size_of_sharedata,
                                       self.judge_no_problem)
 
-    def test_verify_no_problem_4(self):
-        self.basedir = "repairer/Verify/verify_no_problem_4"
+    def test_ok_segment_size(self):
+        self.basedir = "repairer/Verifier/test_ok_segment_size"
         return self._help_test_verify(common._corrupt_segment_size,
                                       self.judge_no_problem)
 
@@ -170,8 +173,8 @@ class Verifier(GridTestMixin, unittest.TestCase, RepairTestMixin):
         self.failUnless(len(data['servers-responding']) == 10, data)
         self.failUnless(len(data['list-corrupt-shares']) == 0, data)
 
-    def test_verify_server_visible_corruption(self):
-        self.basedir = "repairer/Verify/verify_server_visible_corruption"
+    def test_corrupt_file_verno(self):
+        self.basedir = "repairer/Verifier/corrupt_file_verno"
         return self._help_test_verify(common._corrupt_file_version_number,
                                       self.judge_visible_corruption)
 
@@ -194,8 +197,8 @@ class Verifier(GridTestMixin, unittest.TestCase, RepairTestMixin):
         self.failUnlessEqual(len(data['list-incompatible-shares']), 1)
         self.failUnlessEqual(data['count-incompatible-shares'], 1)
 
-    def test_verify_share_version_incompatibility(self):
-        self.basedir = "repairer/Verify/verify_share_version_incompatibility"
+    def test_corrupt_share_verno(self):
+        self.basedir = "repairer/Verifier/corrupt_share_verno"
         return self._help_test_verify(common._corrupt_sharedata_version_number,
                                       self.judge_share_version_incompatibility)
 
@@ -216,82 +219,143 @@ class Verifier(GridTestMixin, unittest.TestCase, RepairTestMixin):
         self.failUnlessEqual(len(data['servers-responding']), 10)
         self.failUnlessEqual(len(data['sharemap']), 9)
 
-    def test_verify_server_invisible_corruption_1(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_1"
+    def test_corrupt_sharedata_offset(self):
+        self.basedir = "repairer/Verifier/corrupt_sharedata_offset"
         return self._help_test_verify(common._corrupt_offset_of_sharedata,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_2(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_2"
+    def test_corrupt_ueb_offset(self):
+        self.basedir = "repairer/Verifier/corrupt_ueb_offset"
         return self._help_test_verify(common._corrupt_offset_of_uri_extension,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_3(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_3"
+    def test_corrupt_ueb_offset_shortread(self):
+        self.basedir = "repairer/Verifier/corrupt_ueb_offset_shortread"
         return self._help_test_verify(common._corrupt_offset_of_uri_extension_to_force_short_read,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_4(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_4"
+    def test_corrupt_sharedata(self):
+        self.basedir = "repairer/Verifier/corrupt_sharedata"
         return self._help_test_verify(common._corrupt_share_data,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_5(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_5"
+    def test_corrupt_ueb_length(self):
+        self.basedir = "repairer/Verifier/corrupt_ueb_length"
         return self._help_test_verify(common._corrupt_length_of_uri_extension,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_6(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_6"
+    def test_corrupt_ueb(self):
+        self.basedir = "repairer/Verifier/corrupt_ueb"
         return self._help_test_verify(common._corrupt_uri_extension,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_offset_of_block_hashtree_to_truncate_crypttext_hashtree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_offset_of_block_hashtree_to_truncate_crypttext_hashtree"
+    def test_truncate_crypttext_hashtree(self):
+        # change the start of the block hashtree, to truncate the preceding
+        # crypttext hashtree
+        self.basedir = "repairer/Verifier/truncate_crypttext_hashtree"
         return self._help_test_verify(common._corrupt_offset_of_block_hashes_to_truncate_crypttext_hashes,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_offset_of_block_hashtree_to_truncate_crypttext_hashtree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_offset_of_block_hashtree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_offset_of_block_hashtree"
+    def test_corrupt_block_hashtree_offset(self):
+        self.basedir = "repairer/Verifier/corrupt_block_hashtree_offset"
         return self._help_test_verify(common._corrupt_offset_of_block_hashes,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_offset_of_block_hashtree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_sharedata_plausible_version(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_sharedata_plausible_version"
+    def test_wrong_share_verno(self):
+        self.basedir = "repairer/Verifier/wrong_share_verno"
         return self._help_test_verify(common._corrupt_sharedata_version_number_to_plausible_version,
                                       self.judge_invisible_corruption)
 
-    def test_verify_server_invisible_corruption_offset_of_share_hashtree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_offset_of_share_hashtree"
+    def test_corrupt_share_hashtree_offset(self):
+        self.basedir = "repairer/Verifier/corrupt_share_hashtree_offset"
         return self._help_test_verify(common._corrupt_offset_of_share_hashes,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_offset_of_share_hashtree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_offset_of_ciphertext_hashtree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_offset_of_ciphertext_hashtree"
+    def test_corrupt_crypttext_hashtree_offset(self):
+        self.basedir = "repairer/Verifier/corrupt_crypttext_hashtree_offset"
         return self._help_test_verify(common._corrupt_offset_of_ciphertext_hash_tree,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_offset_of_ciphertext_hashtree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_cryptext_hash_tree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_cryptext_hash_tree"
+    def test_corrupt_crypttext_hashtree(self):
+        self.basedir = "repairer/Verifier/corrupt_crypttext_hashtree"
         return self._help_test_verify(common._corrupt_crypttext_hash_tree,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_cryptext_hash_tree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_block_hash_tree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_block_hash_tree"
+    def test_corrupt_crypttext_hashtree_byte_x221(self):
+        self.basedir = "repairer/Verifier/corrupt_crypttext_hashtree_byte_9_bit_7"
+        return self._help_test_verify(common._corrupt_crypttext_hash_tree_byte_x221,
+                                      self.judge_invisible_corruption, debug=True)
+
+    def test_corrupt_block_hashtree(self):
+        self.basedir = "repairer/Verifier/corrupt_block_hashtree"
         return self._help_test_verify(common._corrupt_block_hashes,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_block_hash_tree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
 
-    def test_verify_server_invisible_corruption_share_hash_tree_TODO(self):
-        self.basedir = "repairer/Verify/verify_server_invisible_corruption_share_hash_tree"
+    def test_corrupt_share_hashtree(self):
+        self.basedir = "repairer/Verifier/corrupt_share_hashtree"
         return self._help_test_verify(common._corrupt_share_hashes,
                                       self.judge_invisible_corruption)
-    test_verify_server_invisible_corruption_share_hash_tree_TODO.todo = "Verifier doesn't yet properly detect this kind of corruption."
+
+    # TODO: the Verifier should decode to ciphertext and check it against the
+    # crypttext-hash-tree. Check this by constructing a bogus file, in which
+    # the crypttext-hash-tree is modified after encoding is done, but before
+    # the UEB is finalized. The Verifier should see a valid
+    # crypttext-hash-tree but then the ciphertext should show up as invalid.
+    # Normally this could only be triggered by a bug in FEC decode.
+
+    def OFF_test_each_byte(self):
+        # this test takes 140s to run on my laptop, and doesn't have any
+        # actual asserts, so it's commented out. It corrupts each byte of the
+        # share in sequence, and checks to see which ones the Verifier
+        # catches and which it misses. Ticket #819 contains details: there
+        # are several portions of the share that are unused, for which
+        # corruption is not supposed to be caught.
+        #
+        # If the test ran quickly, we could use the share size to compute the
+        # offsets of these unused portions and assert that everything outside
+        # of them was detected. We could then replace the rest of
+        # Verifier.test_* (which takes 16s to run on my laptop) with this
+        # one.
+        self.basedir = "repairer/Verifier/each_byte"
+        self.set_up_grid(num_clients=2)
+        d = self.upload_and_stash()
+        def _grab_sh0(res):
+            self.sh0_file = [sharefile
+                             for (shnum, serverid, sharefile)
+                             in self.find_shares(self.uri)
+                             if shnum == 0][0]
+            self.sh0_orig = open(self.sh0_file, "rb").read()
+        d.addCallback(_grab_sh0)
+        def _fix_sh0(res):
+            f = open(self.sh0_file, "wb")
+            f.write(self.sh0_orig)
+            f.close()
+        def _corrupt(ign, which):
+            def _corruptor(s, debug=False):
+                return s[:which] + chr(ord(s[which])^0x01) + s[which+1:]
+            self.corrupt_shares_numbered(self.uri, [0], _corruptor)
+        results = {}
+        def _did_check(vr, i):
+            #print "corrupt %d: healthy=%s" % (i, vr.is_healthy())
+            results[i] = vr.is_healthy()
+        def _start(ign):
+            d = defer.succeed(None)
+            for i in range(len(self.sh0_orig)):
+                d.addCallback(_corrupt, i)
+                d.addCallback(lambda ign:
+                              self.c1_filenode.check(Monitor(), verify=True))
+                d.addCallback(_did_check, i)
+                d.addCallback(_fix_sh0)
+            return d
+        d.addCallback(_start)
+        def _show_results(ign):
+            f = open("test_each_byte_output", "w")
+            for i in sorted(results.keys()):
+                print >>f, "%d: %s" % (i, results[i])
+            f.close()
+            print "Please look in _trial_temp/test_each_byte_output for results"
+        d.addCallback(_show_results)
+        return d
 
 # We'll allow you to pass this test even if you trigger thirty-five times as
 # many block sends and disk writes as would be optimal.
@@ -424,7 +488,7 @@ class Repairer(GridTestMixin, unittest.TestCase, RepairTestMixin,
         d.addCallback(lambda ignored:
                       self.shouldFail(NotEnoughSharesError, "then_download",
                                       None,
-                                      self.c1_filenode.download_to_data))
+                                      download_to_data, self.c1_filenode))
 
         d.addCallback(lambda ignored:
                       self.shouldFail(NotEnoughSharesError, "then_repair",
@@ -495,14 +559,14 @@ class Repairer(GridTestMixin, unittest.TestCase, RepairTestMixin,
 
         d.addCallback(lambda ignored:
                       self.delete_shares_numbered(self.uri, range(3, 10+1)))
-        d.addCallback(lambda ignored: self.c1_filenode.download_to_data())
+        d.addCallback(lambda ignored: download_to_data(self.c1_filenode))
         d.addCallback(lambda newdata:
                       self.failUnlessEqual(newdata, common.TEST_DATA))
         return d
 
     def test_repair_from_deletion_of_7(self):
         """ Repair replaces seven shares that got deleted. """
-        self.basedir = "repairer/Repairer/repair_from_deletion_of_1"
+        self.basedir = "repairer/Repairer/repair_from_deletion_of_7"
         self.set_up_grid(num_clients=2)
         d = self.upload_and_stash()
         d.addCallback(lambda ignored:
@@ -540,7 +604,7 @@ class Repairer(GridTestMixin, unittest.TestCase, RepairTestMixin,
 
         d.addCallback(lambda ignored:
                       self.delete_shares_numbered(self.uri, range(3, 10+1)))
-        d.addCallback(lambda ignored: self.c1_filenode.download_to_data())
+        d.addCallback(lambda ignored: download_to_data(self.c1_filenode))
         d.addCallback(lambda newdata:
                       self.failUnlessEqual(newdata, common.TEST_DATA))
         return d
@@ -593,7 +657,7 @@ class Repairer(GridTestMixin, unittest.TestCase, RepairTestMixin,
             return res
         d.addCallback(_stash_it)
         def _put_it_all_back(ignored):
-            self.replace_shares(stash[0], storage_index=self.uri.storage_index)
+            self.replace_shares(stash[0], storage_index=self.uri.get_storage_index())
             return ignored
 
         def _repair_from_corruption(shnum, corruptor_func):
