@@ -13,20 +13,27 @@ from allmydata.test import common_util
 import allmydata
 
 bintahoe = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(allmydata.__file__))), 'bin', 'tahoe')
-if sys.platform == "win32":
+if sys.platform == "win32":  # TODO: should this include cygwin?
     bintahoe += ".exe"
 
-class SkipOnCygwinMixin:
-    def skip_on_cygwin(self):
+
+class SkipMixin:
+    def skip_if_cannot_run_bintahoe(self):
         if "cygwin" in sys.platform.lower():
             raise unittest.SkipTest("We don't know how to make this test work on cygwin: spawnProcess seems to hang forever. We don't know if 'bin/tahoe start' can be run on cygwin.")
-
-class TheRightCode(common_util.SignalMixin, unittest.TestCase,
-                   SkipOnCygwinMixin):
-    def test_path(self):
-        self.skip_on_cygwin()
         if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
+            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location (%s), and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of Tahoe. Perhaps running 'setup.py build' again will help." % (bintahoe,))
+
+    def skip_if_cannot_daemonize(self):
+        self.skip_if_cannot_run_bintahoe()
+        if runtime.platformType == "win32":
+            # twistd on windows doesn't daemonize. cygwin should work normally.
+            raise unittest.SkipTest("twistd does not fork under windows")
+
+
+class TheRightCode(common_util.SignalMixin, unittest.TestCase, SkipMixin):
+    def test_path(self):
+        self.skip_if_cannot_run_bintahoe()
         d = utils.getProcessOutputAndValue(bintahoe, args=["--version-and-path"], env=os.environ)
         def _cb(res):
             out, err, rc_or_sig = res
@@ -41,8 +48,9 @@ class TheRightCode(common_util.SignalMixin, unittest.TestCase,
         d.addCallback(_cb)
         return d
 
+
 class CreateNode(unittest.TestCase):
-    # exercise "tahoe create-client", create-introducer,
+    # exercise "tahoe create-node", create-introducer,
     # create-key-generator, and create-stats-gatherer, by calling the
     # corresponding code as a subroutine.
 
@@ -56,10 +64,9 @@ class CreateNode(unittest.TestCase):
         rc = runner.runner(argv, stdout=out, stderr=err)
         return rc, out.getvalue(), err.getvalue()
 
-    def test_client(self):
-        basedir = self.workdir("test_client")
-        c1 = os.path.join(basedir, "c1")
-        argv = ["--quiet", "create-client", "--basedir", c1]
+    def do_create(self, command, basedir):
+        c1 = os.path.join(basedir, command + "-c1")
+        argv = ["--quiet", command, "--basedir", c1]
         rc, out, err = self.run_tahoe(argv)
         self.failUnlessEqual(err, "")
         self.failUnlessEqual(out, "")
@@ -67,6 +74,16 @@ class CreateNode(unittest.TestCase):
         self.failUnless(os.path.exists(c1))
         self.failUnless(os.path.exists(os.path.join(c1, "tahoe-client.tac")))
 
+        # tahoe.cfg should exist, and should have storage enabled for
+        # 'create-node', and disabled for 'create-client'.
+        tahoe_cfg = os.path.join(c1, "tahoe.cfg")
+        self.failUnless(os.path.exists(tahoe_cfg))
+        content = open(tahoe_cfg).read()
+        if command == "create-client":
+            self.failUnless("\n[storage]\nenabled = false\n" in content)
+        else:
+            self.failUnless("\n[storage]\nenabled = true\n" in content)
+
         # creating the client a second time should be rejected
         rc, out, err = self.run_tahoe(argv)
         self.failIfEqual(rc, 0, str((out, err, rc)))
@@ -79,18 +96,27 @@ class CreateNode(unittest.TestCase):
             self.failIf(re.search("[\S][^\.!?]$", line), (line,))
 
         # test that the non --basedir form works too
-        c2 = os.path.join(basedir, "c2")
-        argv = ["--quiet", "create-client", c2]
+        c2 = os.path.join(basedir, command + "c2")
+        argv = ["--quiet", command, c2]
         rc, out, err = self.run_tahoe(argv)
         self.failUnless(os.path.exists(c2))
         self.failUnless(os.path.exists(os.path.join(c2, "tahoe-client.tac")))
 
         # make sure it rejects too many arguments
-        argv = ["create-client", "basedir", "extraarg"]
+        argv = [command, "basedir", "extraarg"]
         self.failUnlessRaises(usage.UsageError,
                               runner.runner, argv,
                               run_by_human=False)
 
+    def test_node(self):
+        basedir = self.workdir("test_node")
+        self.do_create("create-node", basedir)
+
+    def test_client(self):
+        # create-client should behave like create-node --no-storage.
+        basedir = self.workdir("test_client")
+        self.do_create("create-client", basedir)
+
     def test_introducer(self):
         basedir = self.workdir("test_introducer")
         c1 = os.path.join(basedir, "c1")
@@ -213,7 +239,7 @@ class CreateNode(unittest.TestCase):
 
 
 class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
-              SkipOnCygwinMixin):
+              SkipMixin):
     # exercise "tahoe start", for both introducer, client node, and
     # key-generator, by spawning "tahoe start" as a subprocess. This doesn't
     # get us figleaf-based line-level coverage, but it does a better job of
@@ -232,12 +258,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return basedir
 
     def test_introducer(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_introducer")
         c1 = os.path.join(basedir, "c1")
         HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
@@ -332,22 +353,79 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
             return res
         d.addBoth(_remove_hotline)
         return d
-    test_introducer.timeout = 240 # This hit the 120-second timeout on "François Lenny-armv5tel"
+    test_introducer.timeout = 480 # This hit the 120-second timeout on "François Lenny-armv5tel", then it hit a 240-second timeout on our feisty2.5 buildslave: http://allmydata.org/buildbot/builders/feisty2.5/builds/2381/steps/test/logs/test.log
 
-    def test_client(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
-        basedir = self.workdir("test_client")
+    def test_client_no_noise(self):
+        self.skip_if_cannot_daemonize()
+        import pkg_resources
+        try:
+            pkg_resources.require("Twisted>=9.0.0")
+        except pkg_resources.VersionConflict:
+            raise unittest.SkipTest("We pass this test only with Twisted >= v9.0.0")
+        basedir = self.workdir("test_client_no_noise")
         c1 = os.path.join(basedir, "c1")
         HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
         TWISTD_PID_FILE = os.path.join(c1, "twistd.pid")
         PORTNUMFILE = os.path.join(c1, "client.port")
 
         d = utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "create-client", "--basedir", c1, "--webport", "0"], env=os.environ)
+        def _cb(res):
+            out, err, rc_or_sig = res
+            errstr = "cc=%d, OUT: '%s', ERR: '%s'" % (rc_or_sig, out, err)
+            assert rc_or_sig == 0, errstr
+            self.failUnlessEqual(rc_or_sig, 0)
+            # By writing this file, we get forty seconds before the client will exit. This insures
+            # that even if the 'stop' command doesn't work (and the test fails), the client should
+            # still terminate.
+            open(HOTLINE_FILE, "w").write("")
+            open(os.path.join(c1, "introducer.furl"), "w").write("pb://xrndsskn2zuuian5ltnxrte7lnuqdrkz@127.0.0.1:55617/introducer\n")
+            # now it's safe to start the node
+        d.addCallback(_cb)
+
+        def _start(res):
+            return utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "start", c1], env=os.environ)
+        d.addCallback(_start)
+
+        def _cb2(res):
+            out, err, rc_or_sig = res
+            errstr = "cc=%d, OUT: '%s', ERR: '%s'" % (rc_or_sig, out, err)
+            open(HOTLINE_FILE, "w").write("")
+            self.failUnlessEqual(rc_or_sig, 0, errstr)
+            self.failUnlessEqual(out, "", errstr) # If you emit noise, you fail this test.
+            self.failUnlessEqual(err, "", errstr)
+
+            # the parent (twistd) has exited. However, twistd writes the pid
+            # from the child, not the parent, so we can't expect twistd.pid
+            # to exist quite yet.
+
+            # the node is running, but it might not have made it past the
+            # first reactor turn yet, and if we kill it too early, it won't
+            # remove the twistd.pid file. So wait until it does something
+            # that we know it won't do until after the first turn.
+        d.addCallback(_cb2)
+
+        def _node_has_started():
+            return os.path.exists(PORTNUMFILE)
+        d.addCallback(lambda res: self.poll(_node_has_started))
+
+        # now we can kill it. TODO: On a slow machine, the node might kill
+        # itself before we get a chance too, especially if spawning the
+        # 'tahoe stop' command takes a while.
+        def _stop(res):
+            self.failUnless(os.path.exists(TWISTD_PID_FILE), (TWISTD_PID_FILE, os.listdir(os.path.dirname(TWISTD_PID_FILE))))
+            return utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "stop", c1], env=os.environ)
+        d.addCallback(_stop)
+        return d
+
+    def test_client(self):
+        self.skip_if_cannot_daemonize()
+        basedir = self.workdir("test_client")
+        c1 = os.path.join(basedir, "c1")
+        HOTLINE_FILE = os.path.join(c1, "suicide_prevention_hotline")
+        TWISTD_PID_FILE = os.path.join(c1, "twistd.pid")
+        PORTNUMFILE = os.path.join(c1, "client.port")
+
+        d = utils.getProcessOutputAndValue(bintahoe, args=["--quiet", "create-node", "--basedir", c1, "--webport", "0"], env=os.environ)
         def _cb(res):
             out, err, rc_or_sig = res
             self.failUnlessEqual(rc_or_sig, 0)
@@ -438,9 +516,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return d
 
     def test_baddir(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_baddir")
         fileutil.make_dirs(basedir)
 
@@ -474,12 +550,7 @@ class RunNode(common_util.SignalMixin, unittest.TestCase, pollmixin.PollMixin,
         return d
 
     def test_keygen(self):
-        self.skip_on_cygwin()
-        if not os.path.exists(bintahoe):
-            raise unittest.SkipTest("The bin/tahoe script isn't to be found in the expected location, and I don't want to test a 'tahoe' executable that I find somewhere else, in case it isn't the right executable for this version of tahoe.")
-        if runtime.platformType == "win32":
-            # twistd on windows doesn't daemonize. cygwin works normally.
-            raise unittest.SkipTest("twistd does not fork under windows")
+        self.skip_if_cannot_daemonize()
         basedir = self.workdir("test_keygen")
         c1 = os.path.join(basedir, "c1")
         TWISTD_PID_FILE = os.path.join(c1, "twistd.pid")
