@@ -8,9 +8,9 @@ from twisted.internet import defer
 from foolscap.api import fireEventually
 
 import allmydata # for __full_version__
-from allmydata import uri, monitor
+from allmydata import uri, monitor, client
 from allmydata.immutable import upload
-from allmydata.interfaces import IFileURI, FileTooLargeError, NoSharesError, \
+from allmydata.interfaces import FileTooLargeError, NoSharesError, \
      NotEnoughSharesError
 from allmydata.util.assertutil import precondition
 from allmydata.util.deferredutil import DeferredListShouldSucceed
@@ -162,7 +162,7 @@ class FakeBucketWriter:
         self.closed = True
 
     def remote_abort(self):
-        log.err("uh oh, I was asked to abort")
+        log.err(RuntimeError("uh oh, I was asked to abort"))
 
 class FakeClient:
     DEFAULT_ENCODING_PARAMETERS = {"k":25,
@@ -187,11 +187,7 @@ class FakeClient:
         return self.DEFAULT_ENCODING_PARAMETERS
     def get_storage_broker(self):
         return self.storage_broker
-
-    def get_renewal_secret(self):
-        return ""
-    def get_cancel_secret(self):
-        return ""
+    _secret_holder = client.SecretHolder("lease secret", "convergence secret")
 
 class GotTooFarError(Exception):
     pass
@@ -254,15 +250,15 @@ class GoodServer(unittest.TestCase, ShouldFailMixin):
         self.node.DEFAULT_ENCODING_PARAMETERS = p
 
     def _check_small(self, newuri, size):
-        u = IFileURI(newuri)
+        u = uri.from_string(newuri)
         self.failUnless(isinstance(u, uri.LiteralFileURI))
         self.failUnlessEqual(len(u.data), size)
 
     def _check_large(self, newuri, size):
-        u = IFileURI(newuri)
+        u = uri.from_string(newuri)
         self.failUnless(isinstance(u, uri.CHKFileURI))
-        self.failUnless(isinstance(u.storage_index, str))
-        self.failUnlessEqual(len(u.storage_index), 16)
+        self.failUnless(isinstance(u.get_storage_index(), str))
+        self.failUnlessEqual(len(u.get_storage_index()), 16)
         self.failUnless(isinstance(u.key, str))
         self.failUnlessEqual(len(u.key), 16)
         self.failUnlessEqual(u.size, size)
@@ -384,10 +380,10 @@ class ServerErrors(unittest.TestCase, ShouldFailMixin):
         self.u.parent = self.node
 
     def _check_large(self, newuri, size):
-        u = IFileURI(newuri)
+        u = uri.from_string(newuri)
         self.failUnless(isinstance(u, uri.CHKFileURI))
-        self.failUnless(isinstance(u.storage_index, str))
-        self.failUnlessEqual(len(u.storage_index), 16)
+        self.failUnless(isinstance(u.get_storage_index(), str))
+        self.failUnlessEqual(len(u.get_storage_index()), 16)
         self.failUnless(isinstance(u.key, str))
         self.failUnlessEqual(len(u.key), 16)
         self.failUnlessEqual(u.size, size)
@@ -472,10 +468,10 @@ class PeerSelection(unittest.TestCase):
         return DATA[:size]
 
     def _check_large(self, newuri, size):
-        u = IFileURI(newuri)
+        u = uri.from_string(newuri)
         self.failUnless(isinstance(u, uri.CHKFileURI))
-        self.failUnless(isinstance(u.storage_index, str))
-        self.failUnlessEqual(len(u.storage_index), 16)
+        self.failUnless(isinstance(u.get_storage_index(), str))
+        self.failUnlessEqual(len(u.get_storage_index()), 16)
         self.failUnless(isinstance(u.key, str))
         self.failUnlessEqual(len(u.key), 16)
         self.failUnlessEqual(u.size, size)
